# -*- coding: utf-8 -*-

from numpy import sin, pi


def comp_surface(self):
    """Compute the Slot total surface (by analytical computation).
    Caution, the bottom of the Slot is an Arc

    Parameters
    ----------
    self : SlotM11
        A SlotM11 object

    Returns
    -------
    S: float
        Slot total surface [m**2]

    """

    point_dict = self._comp_point_coordinate()
    Z1 = point_dict["Z1"]
    Z2 = point_dict["Z2"]

    R1 = abs(Z1)
    R2 = abs(Z2)

    S1 = pi * R1 ** 2 * (self.W0 / (2 * pi))
    S2 = pi * R2 ** 2 * (self.W0 / (2 * pi))

    return abs(S1 - S2)



# -*- coding: utf-8 -*-

from ....Classes.MeshMat import MeshMat
from ....Classes.MeshVTK import MeshVTK
from ....Functions.Structural.conversions import DimError, cart2pol
from numpy import einsum, sqrt, zeros, squeeze, real, imag, sum as np_sum, abs as np_abs


def get_field(
    self,
    label=None,
    index=None,
    indices=None,
    is_rthetaz=False,
    is_radial=False,
    is_normal=False,
    is_rms=False,
    is_center=False,
    is_surf=False,
    args=None,
):
    """Return the solution corresponding to label or an index.

    Parameters
    ----------
    self : MeshSolution
        an MeshSolution object
    label : str
        a label
    index : int
        an index
    indices : list
        list of indices to extract from mesh and field
    is_rthetaz : bool
        cylindrical coordinates
    is_radial : bool
        radial component only
    is_normal : bool
        normal component only (on nodes or centers)
    is_rms : bool
        rms over surface sum((F.n)**2 dS)/S
    is_center : bool
        field at cell centers
    is_surf : bool
        field over outer surface

    Returns
    -------
    result : ndarray
        field

    """

    # Get field
    solution = self.get_solution(label=label, index=index)

    if args is not None:
        field = solution.get_field(args)
    else:
        field = solution.get_field()

    field = squeeze(field)

    # Check dimensions
    shape = field.shape
    is_other_dim = False
    is_1d_input = False
    is_1d_output = False
    if len(shape) != 1 and shape[-1] > 3:
        is_other_dim = True
    if len(shape) == 1 or (len(shape) == 2 and is_other_dim):
        is_1d_input = True
    if is_radial:
        is_rthetaz = True
    if is_radial or is_1d_input or is_normal:
        is_1d_output = True
    if is_rms:
        is_center = True
        is_normal = True
    if is_normal:
        is_surf = True
    if is_rthetaz or is_normal:
        if is_1d_input:
            raise DimError("The field should be 3D")

    # Get mesh if necessary
    if is_center or is_normal or is_rthetaz or is_surf:
        # Get the mesh
        mesh = self.get_mesh(label=label, index=index)
        mesh_pv = mesh.get_mesh_pv(indices=indices)
        if isinstance(mesh, MeshMat):
            mesh_pv = mesh.get_mesh_pv(indices=indices)
            mesh = MeshVTK(mesh=mesh_pv, is_pyvista_mesh=True)
    # Get points coordinates if necessary
    if is_rthetaz:
        points = mesh.get_point(indices=indices)
    # Get normals if necessary
    if is_normal and is_center:
        # Get normals
        normals = mesh.get_normals(indices=indices)
    elif is_normal:
        normals = mesh.get_normals(indices=indices, loc="point")
    # Get cell area if necessary
    if is_rms:
        cell_area = mesh.get_cell_area(indices=indices)

    # 1D case: only cell-center and surf available
    if is_1d_input:
        if is_center or is_surf:
            if is_other_dim:
                # Field has other dimension -> loop over other dimension
                if is_center:
                    Nnodes = mesh.get_points(indices=indices).shape[0]
                else:
                    Nnodes = shape[0]
                result = zeros((Nnodes, shape[1]), dtype=complex)
                for i in range(shape[1]):
                    field_i = field[:, i]
                    # Extract subset of the field if necessary
                    if indices is not None:
                        if len(field_i) != len(indices):
                            field_i = field_i[indices]
                    # Add field to mesh
                    mesh_pv["real"] = real(field_i)
                    mesh_pv["imag"] = imag(field_i)
                    if is_center:
                        # Points to centers
                        mesh_cell = mesh_pv.point_data_to_cell_data()
                    else:
                        mesh_cell = mesh_pv
                    if is_surf:
                        # Extract surface
                        surf = mesh_cell.extract_geometry()
                        field_i = surf["real"] + 1j * surf["imag"]
                    else:
                        field_i = mesh_cell["real"] + 1j * mesh_cell["imag"]
                    # Store in result
                    result[:, i] = field_i
            else:
                # Extract subset of the field if necessary
                if indices is not None:
                    if len(field) != len(indices):
                        field = field[indices]
                # Add field to mesh
                mesh_pv["real"] = real(field)
                mesh_pv["imag"] = imag(field)
                if is_center:
                    # Points to centers
                    mesh_cell = mesh_pv.point_data_to_cell_data()
                else:
                    mesh_cell = mesh_pv
                if is_surf:
                    # Extract surface
                    surf = mesh_cell.extract_geometry()
                    field = surf["real"] + 1j * surf["imag"]
                else:
                    field = mesh_cell["real"] + 1j * mesh_cell["imag"]
                # Store in result
                result = field
        else:
            result = field

    # 3D case
    else:
        if is_other_dim:
            # Field has third dimension -> loop over third dimension
            if is_center:
                Nnodes = mesh.get_normals(indices=indices).shape[0]
            else:
                Nnodes = shape[0]
            if is_1d_output:
                result = zeros((Nnodes, shape[2]), dtype=complex)
            elif is_rms:
                result = zeros(shape[2], dtype=complex)
            else:
                result = zeros((Nnodes, shape[1], shape[2]), dtype=complex)
            for i in range(shape[2]):
                field_i = field[:, :, i]
                # Extract subset of the field if necessary
                if indices is not None and field_i.shape != indices.shape:
                    field_i = field_i[indices]
                # Field to mesh if necessary
                if is_center or is_normal or is_surf:
                    # Add field to mesh
                    mesh_pv["real"] = real(field_i)
                    mesh_pv["imag"] = imag(field_i)
                    if is_center:
                        # Points to centers
                        mesh_cell = mesh_pv.point_data_to_cell_data()
                    else:
                        mesh_cell = mesh_pv
                if is_surf:
                    # Extract surface
                    surf = mesh_cell.extract_geometry()
                    field_i = surf["real"] + 1j * surf["imag"]
                elif is_center:
                    field_i = mesh_cell["real"] + 1j * mesh_cell["imag"]
                # Project on normals if necessary
                if is_normal:
                    field_i = einsum("ij,ij->i", normals, field_i)
                # rms computation
                if is_rms:
                    field_i = sqrt(
                        np_sum(np_abs(field_i ** 2) * cell_area) / np_sum(cell_area)
                    )
                # Coordinates
                if is_rthetaz:
                    field_i = cart2pol(field_i, points)
                if is_radial:
                    field_i = field_i[:, 0]
                # Store in result
                if is_1d_output:
                    result[:, i] = field_i
                elif is_rms:
                    result[i] = field_i
                else:
                    result[:, :, i] = field_i
        else:
            # Extract subset of the field if necessary
            if indices is not None and field_i.shape != indices.shape:
                field = field[indices]
            # Field to mesh if necessary
            if is_center or is_normal or is_surf:
                # Add field to mesh
                mesh_pv["real"] = real(field)
                mesh_pv["imag"] = imag(field)
                if is_center:
                    # Points to centers
                    mesh_cell = mesh_pv.point_data_to_cell_data()
                else:
                    mesh_cell = mesh_pv
            if is_surf:
                # Extract surface
                surf = mesh_cell.extract_geometry()
                field = surf["real"] + 1j * surf["imag"]
            elif is_center:
                field = mesh_cell["real"] + 1j * mesh_cell["imag"]
            # Project on normals if necessary
            if is_normal:
                field = einsum("ij,ij->i", normals, field)
            # Coordinates
            if is_rthetaz:
                field = cart2pol(field, points)
            if is_radial:
                field = field[:, 0]
            # Store in result
            result = field

    return result



from numpy import cos, sin


def set_Id_Iq(self, I0, Phi0):
    """Set Id_ref and Iq_ref according to I0, Phi0"""

    self.Id_ref = I0 * cos(Phi0)
    self.Iq_ref = I0 * sin(Phi0)
    if abs(self.Id_ref) < 1e-10:
        self.Id_ref = 0
    if abs(self.Iq_ref) < 1e-10:
        self.Iq_ref = 0



# -*- coding: utf-8 -*-


def load_columns(self):
    """Method to load the names of the columns data of the Elmer simulation results

    Parameter
    ---------


    Return
    ------

    """
    pass



# -*- coding: utf-8 -*-

from numpy import linspace, meshgrid, pi, repeat, sum as np_sum, zeros, mean
from ....Functions.Winding.comp_cond_function import comp_cond_function


def comp_wind_function(self, angle=None, Na=2048, alpha_mmf0=0, per_a=1):
    """Computation of the winding function for the lamination.
    By convention a tooth is centered on the X axis

    Parameters
    ----------
    self : LamSlotWind
        A LamSlotWind object
    angle : ndarray
        Space discretization to compute the winding functions
    Na : int
        Number of angular points for the winding function (not used if angle is set)
    alpha_mmf0 : float
        Angle to shift the winding function (Default value = 0)
    per_a : int
        Spatial periodicity factor

    Returns
    -------
    wf: ndarray
        Winding function Matrix (qs,Na)
    """

    # Space discretization
    if angle is None:
        angle = linspace(0, pi * 2 / per_a, Na, endpoint=False)
    else:
        Na = angle.size

    qs = self.winding.qs  # number of phases
    # Number of point on rad and tan direction
    Nrad, Ntan = self.winding.get_dim_wind()
    Zs = self.slot.Zs  # Number of slot
    Zs0 = int(Zs / per_a)

    slot_angle = self.slot.comp_angle_active_eq()
    slot_opening = self.slot.comp_angle_opening()

    wind_mat = self.winding.comp_connection_mat(Zs)
    # sum wind_mat along Nlay_rad axis
    wind_mat = np_sum(wind_mat, axis=0)

    # angle of the center of the slots
    # By convention a tooth is centered on the X axis
    alpha_slot = linspace(0, 2 * pi / per_a, Zs0, endpoint=False) + pi / Zs

    # angle of the lay in a slot (Nlay point, end and begin excluded)
    alpha_lay = linspace(-slot_angle / 2, slot_angle / 2, Ntan + 1, endpoint=False)[1:]

    if alpha_mmf0 != 0:
        angle = (angle - alpha_mmf0) % (2 * pi)

    wf = zeros((qs, Na))
    for n in range(Ntan):
        # [Na, Zs]
        Xalpha_slot, Xangle = meshgrid(alpha_slot + alpha_lay[n], angle)

        Xwind = zeros((1, Zs0, qs))
        Xwind[0, :, :] = wind_mat[n, 0:Zs0, :]
        # Extended winding matrix [Na, Zs, qs]
        Xwind = repeat(Xwind, Na, axis=0)
        # Single winding function in every slot and angle for nth layer [Na, Zs]
        Xwf = -0.5 * comp_cond_function(Xalpha_slot, slot_opening, Xangle)

        for q in range(qs):
            # winding function of qth phase (sum over layers) [qs, Na]
            wf[q, :] += np_sum(Xwind[:, :, q] * Xwf, axis=1)

    if per_a > 1:
        wf = wf - mean(wf, axis=1)[:, None]

    return wf



# -*- coding: utf-8 -*-
from numpy import mean, zeros
from ....Functions.Electrical.coordinate_transformation import n2ab


def comp_joule_losses(self, output):
    """Compute the electrical Joule losses

    Parameters
    ----------
    self : Electrical
        an Electrical object
    output : Output
        an Output object
    """
    # TODO utilize loss models instead here
    # compute stator joule losses
    qs = output.simu.machine.stator.winding.qs
    Id = output.elec.Id_ref
    Iq = output.elec.Iq_ref
    Rs = self.parameters["Rs"]

    P_joule_s = qs * Rs * (Id ** 2 + Iq ** 2)

    # compute rotor joule losses
    qr = output.simu.machine.rotor.winding.qs
    p = output.simu.machine.rotor.winding.p
    Rr = self.parameters["Rr_norm"] / self.parameters["norm"] ** 2

    # get the bar currents
    Ir = output.elec.Ir.get_along("time", "phase")["Ir"].T

    # transform rotor current to 2 phase equivalent
    qr_eff = qr // p
    Ir_2ph = zeros([Ir.shape[0], 2])
    for ii in range(p):
        id0 = qr_eff * ii
        id1 = qr_eff * (ii + 1)
        Ir_2ph += n2ab(Ir[:, id0:id1], n=qr_eff) / p

    Ir_mag = abs(Ir_2ph[:, 0] + 1j * Ir_2ph[:, 1])

    P_joule_r = 3 * Rr * mean(Ir_mag ** 2) / 2

    output.elec.Pj_losses = P_joule_s + P_joule_r



# -*- coding: utf-8 -*-


def get_magnet_list(self):
    """Return the list of magnet (including None)

    Parameters
    ----------
    self : HoleMag
        A HoleMag object

    Returns
    -------
    magnet_list : list
        List of Magnet

    """

    magnet_list = list()
    if hasattr(self, "magnet_dict"):
        for value in self.magnet_dict.values():
            magnet_list.append(value)
    else:
        mag_id = 0
        while hasattr(self, "magnet_" + str(mag_id)):
            magnet_list.append(getattr(self, "magnet_" + str(mag_id)))
            mag_id += 1
    return magnet_list



# -*- coding: utf-8 -*-

from ....Methods.Machine.Winding import WindingError


class WindingT1DefMsError(WindingError):
    """

    Parameters
    ----------

    Returns
    -------

    Raises
    ------
    must
        be 0

    """

    pass



# -*- coding: utf-8 -*-

# File generated according to PHoleM50.ui
# WARNING! All changes made in this file will be lost!
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *

from ......GUI.Tools.FloatEdit import FloatEdit
from ......GUI.Dialog.DMatLib.WMatSelect.WMatSelect import WMatSelect

from pyleecan.GUI.Resources import pyleecan_rc


class Ui_PHoleM50(object):
    def setupUi(self, PHoleM50):
        if not PHoleM50.objectName():
            PHoleM50.setObjectName(u"PHoleM50")
        PHoleM50.resize(978, 561)
        PHoleM50.setMinimumSize(QSize(740, 440))
        PHoleM50.setMaximumSize(QSize(16777215, 16777215))
        self.horizontalLayout = QHBoxLayout(PHoleM50)
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.verticalLayout_3 = QVBoxLayout()
        self.verticalLayout_3.setObjectName(u"verticalLayout_3")
        self.img_slot = QLabel(PHoleM50)
        self.img_slot.setObjectName(u"img_slot")
        sizePolicy = QSizePolicy(QSizePolicy.Preferred, QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.img_slot.sizePolicy().hasHeightForWidth())
        self.img_slot.setSizePolicy(sizePolicy)
        self.img_slot.setMinimumSize(QSize(0, 0))
        self.img_slot.setMaximumSize(QSize(16777215, 16777215))
        self.img_slot.setPixmap(
            QPixmap(u":/images/images/MachineSetup/SMHoleMag/HoleM50.png")
        )
        self.img_slot.setScaledContents(True)

        self.verticalLayout_3.addWidget(self.img_slot)

        self.txt_constraint = QTextEdit(PHoleM50)
        self.txt_constraint.setObjectName(u"txt_constraint")
        sizePolicy1 = QSizePolicy(QSizePolicy.Preferred, QSizePolicy.Fixed)
        sizePolicy1.setHorizontalStretch(0)
        sizePolicy1.setVerticalStretch(0)
        sizePolicy1.setHeightForWidth(
            self.txt_constraint.sizePolicy().hasHeightForWidth()
        )
        self.txt_constraint.setSizePolicy(sizePolicy1)
        self.txt_constraint.setMinimumSize(QSize(200, 0))
        self.txt_constraint.setMaximumSize(QSize(16777215, 120))
        self.txt_constraint.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.txt_constraint.setTextInteractionFlags(
            Qt.TextSelectableByKeyboard | Qt.TextSelectableByMouse
        )

        self.verticalLayout_3.addWidget(self.txt_constraint)

        self.horizontalLayout.addLayout(self.verticalLayout_3)

        self.widget = QWidget(PHoleM50)
        self.widget.setObjectName(u"widget")
        self.widget.setMinimumSize(QSize(250, 0))
        self.widget.setMaximumSize(QSize(250, 16777215))
        self.verticalLayout_2 = QVBoxLayout(self.widget)
        self.verticalLayout_2.setObjectName(u"verticalLayout_2")
        self.gridLayout = QGridLayout()
        self.gridLayout.setObjectName(u"gridLayout")
        self.unit_W2 = QLabel(self.widget)
        self.unit_W2.setObjectName(u"unit_W2")

        self.gridLayout.addWidget(self.unit_W2, 7, 2, 1, 1)

        self.unit_W1 = QLabel(self.widget)
        self.unit_W1.setObjectName(u"unit_W1")

        self.gridLayout.addWidget(self.unit_W1, 6, 2, 1, 1)

        self.lf_W0 = FloatEdit(self.widget)
        self.lf_W0.setObjectName(u"lf_W0")

        self.gridLayout.addWidget(self.lf_W0, 5, 1, 1, 1)

        self.unit_H2 = QLabel(self.widget)
        self.unit_H2.setObjectName(u"unit_H2")

        self.gridLayout.addWidget(self.unit_H2, 2, 2, 1, 1)

        self.in_H4 = QLabel(self.widget)
        self.in_H4.setObjectName(u"in_H4")

        self.gridLayout.addWidget(self.in_H4, 4, 0, 1, 1)

        self.lf_H2 = FloatEdit(self.widget)
        self.lf_H2.setObjectName(u"lf_H2")

        self.gridLayout.addWidget(self.lf_H2, 2, 1, 1, 1)

        self.lf_H1 = FloatEdit(self.widget)
        self.lf_H1.setObjectName(u"lf_H1")

        self.gridLayout.addWidget(self.lf_H1, 1, 1, 1, 1)

        self.unit_H1 = QLabel(self.widget)
        self.unit_H1.setObjectName(u"unit_H1")

        self.gridLayout.addWidget(self.unit_H1, 1, 2, 1, 1)

        self.lf_H4 = FloatEdit(self.widget)
        self.lf_H4.setObjectName(u"lf_H4")

        self.gridLayout.addWidget(self.lf_H4, 4, 1, 1, 1)

        self.in_W1 = QLabel(self.widget)
        self.in_W1.setObjectName(u"in_W1")

        self.gridLayout.addWidget(self.in_W1, 6, 0, 1, 1)

        self.lf_H0 = FloatEdit(self.widget)
        self.lf_H0.setObjectName(u"lf_H0")

        self.gridLayout.addWidget(self.lf_H0, 0, 1, 1, 1)

        self.unit_H0 = QLabel(self.widget)
        self.unit_H0.setObjectName(u"unit_H0")

        self.gridLayout.addWidget(self.unit_H0, 0, 2, 1, 1)

        self.lf_W3 = FloatEdit(self.widget)
        self.lf_W3.setObjectName(u"lf_W3")

        self.gridLayout.addWidget(self.lf_W3, 8, 1, 1, 1)

        self.in_H1 = QLabel(self.widget)
        self.in_H1.setObjectName(u"in_H1")

        self.gridLayout.addWidget(self.in_H1, 1, 0, 1, 1)

        self.unit_H4 = QLabel(self.widget)
        self.unit_H4.setObjectName(u"unit_H4")

        self.gridLayout.addWidget(self.unit_H4, 4, 2, 1, 1)

        self.in_H3 = QLabel(self.widget)
        self.in_H3.setObjectName(u"in_H3")

        self.gridLayout.addWidget(self.in_H3, 3, 0, 1, 1)

        self.unit_W0 = QLabel(self.widget)
        self.unit_W0.setObjectName(u"unit_W0")

        self.gridLayout.addWidget(self.unit_W0, 5, 2, 1, 1)

        self.in_W2 = QLabel(self.widget)
        self.in_W2.setObjectName(u"in_W2")

        self.gridLayout.addWidget(self.in_W2, 7, 0, 1, 1)

        self.in_H0 = QLabel(self.widget)
        self.in_H0.setObjectName(u"in_H0")

        self.gridLayout.addWidget(self.in_H0, 0, 0, 1, 1)

        self.unit_H3 = QLabel(self.widget)
        self.unit_H3.setObjectName(u"unit_H3")

        self.gridLayout.addWidget(self.unit_H3, 3, 2, 1, 1)

        self.lf_W1 = FloatEdit(self.widget)
        self.lf_W1.setObjectName(u"lf_W1")

        self.gridLayout.addWidget(self.lf_W1, 6, 1, 1, 1)

        self.lf_H3 = FloatEdit(self.widget)
        self.lf_H3.setObjectName(u"lf_H3")

        self.gridLayout.addWidget(self.lf_H3, 3, 1, 1, 1)

        self.unit_W4 = QLabel(self.widget)
        self.unit_W4.setObjectName(u"unit_W4")

        self.gridLayout.addWidget(self.unit_W4, 9, 2, 1, 1)

        self.in_W0 = QLabel(self.widget)
        self.in_W0.setObjectName(u"in_W0")

        self.gridLayout.addWidget(self.in_W0, 5, 0, 1, 1)

        self.in_H2 = QLabel(self.widget)
        self.in_H2.setObjectName(u"in_H2")

        self.gridLayout.addWidget(self.in_H2, 2, 0, 1, 1)

        self.in_W3 = QLabel(self.widget)
        self.in_W3.setObjectName(u"in_W3")

        self.gridLayout.addWidget(self.in_W3, 8, 0, 1, 1)

        self.lf_W2 = FloatEdit(self.widget)
        self.lf_W2.setObjectName(u"lf_W2")

        self.gridLayout.addWidget(self.lf_W2, 7, 1, 1, 1)

        self.lf_W4 = FloatEdit(self.widget)
        self.lf_W4.setObjectName(u"lf_W4")

        self.gridLayout.addWidget(self.lf_W4, 9, 1, 1, 1)

        self.in_W4 = QLabel(self.widget)
        self.in_W4.setObjectName(u"in_W4")

        self.gridLayout.addWidget(self.in_W4, 9, 0, 1, 1)

        self.unit_W3 = QLabel(self.widget)
        self.unit_W3.setObjectName(u"unit_W3")

        self.gridLayout.addWidget(self.unit_W3, 8, 2, 1, 1)

        self.verticalLayout_2.addLayout(self.gridLayout)

        self.w_mat_0 = WMatSelect(self.widget)
        self.w_mat_0.setObjectName(u"w_mat_0")
        self.w_mat_0.setMinimumSize(QSize(100, 0))

        self.verticalLayout_2.addWidget(self.w_mat_0)

        self.w_mat_1 = WMatSelect(self.widget)
        self.w_mat_1.setObjectName(u"w_mat_1")
        self.w_mat_1.setMinimumSize(QSize(100, 0))

        self.verticalLayout_2.addWidget(self.w_mat_1)

        self.w_mat_2 = WMatSelect(self.widget)
        self.w_mat_2.setObjectName(u"w_mat_2")
        self.w_mat_2.setMinimumSize(QSize(100, 0))

        self.verticalLayout_2.addWidget(self.w_mat_2)

        self.verticalSpacer = QSpacerItem(
            20, 40, QSizePolicy.Minimum, QSizePolicy.Expanding
        )

        self.verticalLayout_2.addItem(self.verticalSpacer)

        self.g_output = QGroupBox(self.widget)
        self.g_output.setObjectName(u"g_output")
        self.g_output.setMinimumSize(QSize(200, 0))
        self.verticalLayout = QVBoxLayout(self.g_output)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.out_slot_surface = QLabel(self.g_output)
        self.out_slot_surface.setObjectName(u"out_slot_surface")

        self.verticalLayout.addWidget(self.out_slot_surface)

        self.out_magnet_surface = QLabel(self.g_output)
        self.out_magnet_surface.setObjectName(u"out_magnet_surface")

        self.verticalLayout.addWidget(self.out_magnet_surface)

        self.out_alpha = QLabel(self.g_output)
        self.out_alpha.setObjectName(u"out_alpha")

        self.verticalLayout.addWidget(self.out_alpha)

        self.out_W5 = QLabel(self.g_output)
        self.out_W5.setObjectName(u"out_W5")

        self.verticalLayout.addWidget(self.out_W5)

        self.verticalLayout_2.addWidget(self.g_output)

        self.horizontalLayout.addWidget(self.widget)

        QWidget.setTabOrder(self.lf_H0, self.lf_H1)
        QWidget.setTabOrder(self.lf_H1, self.lf_H2)
        QWidget.setTabOrder(self.lf_H2, self.lf_H3)
        QWidget.setTabOrder(self.lf_H3, self.lf_W0)
        QWidget.setTabOrder(self.lf_W0, self.lf_W1)
        QWidget.setTabOrder(self.lf_W1, self.lf_W2)
        QWidget.setTabOrder(self.lf_W2, self.lf_W3)
        QWidget.setTabOrder(self.lf_W3, self.lf_W4)

        self.retranslateUi(PHoleM50)

        QMetaObject.connectSlotsByName(PHoleM50)

    # setupUi

    def retranslateUi(self, PHoleM50):
        PHoleM50.setWindowTitle(QCoreApplication.translate("PHoleM50", u"Form", None))
        self.img_slot.setText("")
        self.txt_constraint.setHtml(
            QCoreApplication.translate(
                "PHoleM50",
                u'<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0//EN" "http://www.w3.org/TR/REC-html40/strict.dtd">\n'
                '<html><head><meta name="qrichtext" content="1" /><style type="text/css">\n'
                "p, li { white-space: pre-wrap; }\n"
                "</style></head><body style=\" font-family:'MS Shell Dlg 2'; font-size:7.8pt; font-weight:400; font-style:normal;\">\n"
                '<p align="center" style=" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;"><span style=" font-family:\'MS Shell Dlg 2\'; font-size:12pt; font-weight:600; text-decoration: underline;">Constraints :</span></p>\n'
                '<p align="center" style=" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;"><span style=" font-family:\'MS Shell Dlg 2\'; font-size:14pt;">H2 &lt; H3</span></p>\n'
                '<p align="center" style=" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;"><span style=" font-family:\'MS Shell Dlg 2'
                "'; font-size:14pt;\">W1 &lt; W0</span></p>\n"
                '<p align="center" style=" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;"><span style=" font-family:\'MS Shell Dlg 2\'; font-size:14pt;">H1 &lt; H0 </span></p></body></html>',
                None,
            )
        )
        self.unit_W2.setText(QCoreApplication.translate("PHoleM50", u"m", None))
        self.unit_W1.setText(QCoreApplication.translate("PHoleM50", u"m", None))
        self.unit_H2.setText(QCoreApplication.translate("PHoleM50", u"m", None))
        self.in_H4.setText(QCoreApplication.translate("PHoleM50", u"H4", None))
        self.unit_H1.setText(QCoreApplication.translate("PHoleM50", u"m", None))
        self.in_W1.setText(QCoreApplication.translate("PHoleM50", u"W1", None))
        self.unit_H0.setText(QCoreApplication.translate("PHoleM50", u"m", None))
        self.in_H1.setText(QCoreApplication.translate("PHoleM50", u"H1", None))
        self.unit_H4.setText(QCoreApplication.translate("PHoleM50", u"m", None))
        self.in_H3.setText(QCoreApplication.translate("PHoleM50", u"H3", None))
        self.unit_W0.setText(QCoreApplication.translate("PHoleM50", u"m", None))
        self.in_W2.setText(QCoreApplication.translate("PHoleM50", u"W2", None))
        self.in_H0.setText(QCoreApplication.translate("PHoleM50", u"H0", None))
        self.unit_H3.setText(QCoreApplication.translate("PHoleM50", u"m", None))
        self.unit_W4.setText(QCoreApplication.translate("PHoleM50", u"m", None))
        self.in_W0.setText(QCoreApplication.translate("PHoleM50", u"W0", None))
        self.in_H2.setText(QCoreApplication.translate("PHoleM50", u"H2", None))
        self.in_W3.setText(QCoreApplication.translate("PHoleM50", u"W3", None))
        self.lf_W4.setText("")
        self.in_W4.setText(QCoreApplication.translate("PHoleM50", u"W4", None))
        self.unit_W3.setText(QCoreApplication.translate("PHoleM50", u"m", None))
        self.g_output.setTitle(QCoreApplication.translate("PHoleM50", u"Output", None))
        self.out_slot_surface.setText(
            QCoreApplication.translate("PHoleM50", u"Slot suface (2 part) : ?", None)
        )
        self.out_magnet_surface.setText(
            QCoreApplication.translate("PHoleM50", u"Single Magnet surface : ?", None)
        )
        self.out_alpha.setText(
            QCoreApplication.translate("PHoleM50", u"Alpha : ?", None)
        )
        self.out_W5.setText(QCoreApplication.translate("PHoleM50", u"W5 : ?", None))

    # retranslateUi



from ....Methods.Geometry.Segment import *


def check(self):
    """assert that the line is correct (begin != end)

    Parameters
    ----------
    self : Segment
        A Segment object

    Returns
    -------
    None

    Raises
    ------
    PointSegmentError
        The beginning point and the ending point of an Segment
        can't be the same

    """
    if self.begin == self.end or (abs(self.begin) == 0 and abs(self.end) == 0):
        raise PointSegmentError(
            "The beginning point and the ending point of " "a Segment can't be the same"
        )



from ....Classes.Simulation import Simulation
from ....Classes.InputCurrent import InputCurrent


def get_input_list(self):
    """Return a list of InputCurrent to set the Operating point"""

    # Check that the object has the correct type
    assert isinstance(self.parent, Simulation)
    ref_simu = self.parent
    assert isinstance(ref_simu.input, InputCurrent)
    N_simu = self.OP_matrix.shape[0]

    # Generate initial input_list
    ref_input = ref_simu.input.copy()
    input_list = [ref_input.copy() for ii in range(N_simu)]

    # Set default time vector (enforce definition Nt_tot, Nrev)
    if ref_input.Nt_tot is None:
        Nt_tot = len(ref_input.time.get_data())
    else:
        Nt_tot = ref_input.Nt_tot
    if ref_input.Nrev is None:
        Nrev = 1
    else:
        Nrev = ref_input.Nrev
    # Update OP according to OP_matrix
    for ii in range(N_simu):
        input_list[ii].N0 = self.OP_matrix[ii, 0]
        # Edit time vector
        input_list[ii].time = None
        input_list[ii].Nt_tot = Nt_tot
        input_list[ii].Nrev = Nrev
        if self.type_OP_matrix == 0:  # I0, Phi0
            input_list[ii].set_Id_Iq(
                I0=self.OP_matrix[ii, 1], Phi0=self.OP_matrix[ii, 2]
            )
        else:  # Id/Iq
            input_list[ii].Id_ref = self.OP_matrix[ii, 1]
            input_list[ii].Iq_ref = self.OP_matrix[ii, 2]
        if self.is_torque:
            input_list[ii].Tem_av_ref = self.OP_matrix[ii, 3]

    return input_list



# -*- coding: utf-8 -*-

from numpy import exp, pi

from ...Functions.FEMM.create_FEMM_bar import create_FEMM_bar
from ...Functions.FEMM.create_FEMM_circuit_material import create_FEMM_circuit_material
from ...Functions.FEMM.create_FEMM_magnet import create_FEMM_magnet


def create_FEMM_materials(
    femm,
    machine,
    surf_list,
    Is,
    Ir,
    BHs,
    BHr,
    is_mmfs,
    is_mmfr,
    type_BH_stator,
    type_BH_rotor,
    is_eddies,
    j_t0,
):
    """Add materials in FEMM

    Parameters
    ----------
    femm : FEMMHandler
        client to send command to a FEMM instance
    machine : Machine
        the machine to simulate
    surf_list : list
        List of surface of the machine
    Is : ndarray
        Stator current matrix [A]
    Ir : ndarray
        Rotor current matrix [A]
    BHs: ndarray
        B(H) curve of the stator
    BHr: ndarray
        B(H) curve of the rotor
    is_mmfs : bool
        1 to compute the stator magnetomotive force/stator magnetic field
    is_mmfr : bool
        1 to compute the rotor magnetomotive force / rotor magnetic field
    type_BH_stator: int
        2 Infinite permeability, 1 to use linear B(H) curve according to mur_lin, 0 to use the B(H) curve
    type_BH_rotor: int
        2 Infinite permeability, 1 to use linear B(H) curve according to mur_lin, 0 to use the B(H) curve
    is_eddies : bool
        1 to calculate eddy currents
    jt_0 : int
        Current time step for winding calculation

    Returns
    -------
    Tuple: dict, list
        Dictionary of properties and list containing the name of the circuits created
    """

    prop_dict = dict()  # Initialisation of the dictionnary to return

    rotor = machine.rotor
    stator = machine.stator

    materials = list()
    circuits = list()
    # Starting creation of properties for each surface of the machine
    for surf in surf_list:
        label = surf.label
        if "Lamination_Stator" in label:  # Stator
            if type_BH_stator == 2:
                mu_is = 100000  # Infinite permeability
            else:
                mu_is = stator.mat_type.mag.mur_lin  # Relative
            # Check if the property already exist in FEMM
            if "Stator Iron" not in materials:
                # magnetic permeability
                femm.mi_addmaterial(
                    "Stator Iron", mu_is, mu_is, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0
                )
                materials.append("Stator Iron")
            prop_dict[label] = "Stator Iron"
        elif "Lamination_Rotor" in label:  # Rotor
            # Initialisation from the rotor of the machine
            if type_BH_rotor == 2:
                mu_ir = 100000  # Infinite permeability
            else:
                mu_ir = rotor.mat_type.mag.mur_lin  # Relative
            # Check if the property already exist in FEMM
            if "Rotor Iron" not in materials:
                # magnetic permeability
                femm.mi_addmaterial(
                    "Rotor Iron", mu_ir, mu_ir, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0
                )
                materials.append("Rotor Iron")
            prop_dict[label] = "Rotor Iron"
        elif "Airgap" in label:  # Airgap surface
            if "Airgap" not in materials:
                femm.mi_addmaterial("Airgap", 1, 1, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0)
                materials.append("Airgap")
            prop_dict[label] = "Airgap"
        elif "Ventilation" in label:  # Ventilation
            # Check if the property already exist in FEMM
            if "Air" not in materials:
                femm.mi_addmaterial("Air", 1, 1, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0)
                materials.append("Air")
            prop_dict[label] = "Air"
        elif "Hole_" in label:  # Hole but not HoleMagnet
            # Check if the property already exist in FEMM
            if "Air" not in materials:
                femm.mi_addmaterial("Air", 1, 1, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0)
                materials.append("Air")
            prop_dict[label] = "Air"
        elif "Wind" in label or "Bar" in label:
            I = Is if "Stator" in label else Ir
            is_mmf = is_mmfs if "Stator" in label else is_mmfr
            lam = stator if "Stator" in label else rotor
            prop, materials, circuits = create_FEMM_circuit_material(
                femm, circuits, label, is_eddies, lam, I, is_mmf, j_t0, materials
            )
            prop_dict[label] = prop
        elif "Magnet" in label and "Rotor" in label:  # Rotor Magnet
            prop, materials = create_FEMM_magnet(
                femm, label, is_mmfr, is_eddies, materials, rotor
            )
            prop_dict[label] = prop
        elif "Magnet" in label and "Stator" in label:  # Stator Magnet
            prop, materials = create_FEMM_magnet(
                femm, label, is_mmfs, is_eddies, materials, stator
            )
            prop_dict[label] = prop
        elif "No_mesh" in label:  # Sliding band
            prop_dict[label] = "<No Mesh>"
        elif "Yoke" in label:
            prop_dict[label] = "<No Mesh>"

    # Set Rotor and Stator BH curves (if needed)
    if type_BH_stator == 0:
        for ii in range(BHs.shape[0]):
            femm.mi_addbhpoint("Stator Iron", BHs[ii][1], BHs[ii][0])
    if type_BH_rotor == 0:
        for ii in range(BHr.shape[0]):
            femm.mi_addbhpoint("Rotor Iron", BHr[ii][1], BHr[ii][0])

    return prop_dict, materials, circuits



# -*- coding: utf-8 -*-


def comp_height(self):
    """Compute the height of the conductor

    Parameters
    ----------
    self : CondType11
        A CondType11 object

    Returns
    -------
    H: float
        Height of the conductor [m]

    """

    return (2 * self.Wins_wire + self.Hwire) * self.Nwppc_rad



from numpy import pi


def get_notch_list(self, sym=1):
    """Returns an ordered description of the notches

    Parameters
    ----------
    self : NotchEvenDist
        A NotchEvenDist object
    sym: int
        Number of symmetry

    Returns
    -------
    notch_list : list
        list of dictionary with key: "begin_angle", "end_angle", "obj"
    """

    notch_list = list()
    op = self.notch_shape.comp_angle_opening()

    for ii in range(self.notch_shape.Zs // sym):
        notch_dict = dict()
        notch_dict["begin_angle"] = (
            2 * pi / self.notch_shape.Zs * ii + self.alpha - op / 2
        )
        notch_dict["end_angle"] = (
            2 * pi / self.notch_shape.Zs * ii + self.alpha + op / 2
        )
        notch_dict["obj"] = self.notch_shape
        notch_list.append(notch_dict)

    return notch_list



# -*- coding: utf-8 -*-

from matplotlib.patches import Patch
from matplotlib.pyplot import axis, legend

from ....Functions.Winding.find_wind_phase_color import find_wind_phase_color
from ....Functions.Winding.gen_phase_list import gen_name
from ....Functions.init_fig import init_fig
from ....definitions import config_dict
from ....Classes.WindingSC import WindingSC
from ....Classes.Winding import Winding

PHASE_COLORS = config_dict["PLOT"]["COLOR_DICT"]["PHASE_COLORS"]
ROTOR_COLOR = config_dict["PLOT"]["COLOR_DICT"]["ROTOR_COLOR"]
STATOR_COLOR = config_dict["PLOT"]["COLOR_DICT"]["STATOR_COLOR"]


def plot(
    self,
    fig=None,
    ax=None,
    is_lam_only=False,
    sym=1,
    alpha=0,
    delta=0,
    is_edge_only=False,
    is_display=True,
    is_show_fig=True,
):
    """Plot the Lamination in a matplotlib fig

    Parameters
    ----------
    self : LamSlotWind
        A LamSlotWind object
    fig : Matplotlib.figure.Figure
        existing figure to use if None create a new one
    ax : Matplotlib.axes.Axes object
        Axis on which to plot the data
    is_lam_only : bool
        True to plot only the lamination (remove the Winding)
    sym : int
        Symmetry factor (1= full machine, 2= half of the machine...)
    alpha : float
        Angle for rotation [rad]
    delta : complex
        Complex value for translation
    is_edge_only: bool
        To plot transparent Patches
    is_display : bool
        False to return the patches
    is_show_fig : bool
        To call show at the end of the method
    Returns
    -------
    patches : list
        List of Patches
    """
    if self.is_stator:
        color_lam = STATOR_COLOR
    else:
        color_lam = ROTOR_COLOR

    # Get the LamSlot surface(s)
    surf_list = self.build_geometry(sym=sym, alpha=alpha, delta=delta)

    patches = list()
    # getting the number of phases and winding connection matrix
    if self.winding is not None and not type(self.winding) is Winding:
        if isinstance(self.winding, WindingSC):  # plot only one phase for WindingSC
            wind_mat = None
            qs = 1
        else:
            try:
                wind_mat = self.winding.comp_connection_mat(self.get_Zs())
                qs = self.winding.qs
            except:
                wind_mat = None
                qs = 1
    else:
        wind_mat = None
        qs = 1

    for surf in surf_list:
        if surf.label is not None and "Lamination" in surf.label:
            patches.extend(surf.get_patches(color_lam, is_edge_only=is_edge_only))
        elif "Wind" in surf.label or "Bar" in surf.label:
            if not is_lam_only:
                color = find_wind_phase_color(wind_mat=wind_mat, label=surf.label)
                patches.extend(surf.get_patches(color=color, is_edge_only=is_edge_only))
        else:
            patches.extend(surf.get_patches(is_edge_only=is_edge_only))

    if is_display:
        # Display the result
        (fig, axes, patch_leg, label_leg) = init_fig(fig=fig, ax=ax, shape="rectangle")
        axes.set_xlabel("(m)")
        axes.set_ylabel("(m)")
        for patch in patches:
            axes.add_patch(patch)
        # Axis Setup
        axes.axis("equal")

        # The Lamination is centered in the figure
        Lim = self.Rext * 1.5
        axes.set_xlim(-Lim, Lim)
        axes.set_ylim(-Lim, Lim)

        # Add the legend
        if not is_edge_only:
            if self.is_stator and "Stator" not in label_leg:
                patch_leg.append(Patch(color=STATOR_COLOR))
                label_leg.append("Stator")
                axes.set_title("Stator with Winding")
            elif not self.is_stator and "Rotor" not in label_leg:
                patch_leg.append(Patch(color=ROTOR_COLOR))
                label_leg.append("Rotor")
                axes.set_title("Rotor with Winding")
            # Add the winding legend only if needed
            if not is_lam_only:
                if self.is_stator:
                    prefix = "Stator "
                else:
                    prefix = "Rotor "
                if isinstance(self.winding, WindingSC):
                    patch_leg.append(Patch(color=PHASE_COLORS[0]))
                    label_leg.append(prefix + "Bar")
                elif self.winding is not None and not type(self.winding) is Winding:
                    phase_name = [prefix + n for n in gen_name(qs, is_add_phase=True)]
                    for ii in range(qs):
                        if not phase_name[ii] in label_leg:
                            # Avoid adding twice the same label
                            index = ii % len(PHASE_COLORS)
                            patch_leg.append(Patch(color=PHASE_COLORS[index]))
                            label_leg.append(phase_name[ii])
            legend(patch_leg, label_leg)
        if is_show_fig:
            fig.show()
    else:
        return patches



from ....Classes.WindingSC import WindingSC
from numpy import array


def comp_I_mag(self, time, is_stator, phase=None):
    """Compute the current on the given lamination and time vector to use it in Magnetics model
    Phase currents are divided by the number of parallel circuits per pole
    and per phase to account for actual current in slot conductors

    Parameters
    ----------
    self : OutElec
        an OutElec object
    time : ndarray
        Time vector on which to interpolate currents stored in OutElec
    is_stator: bool
        True if lamination is stator
    per_a: int
        (Anti-)periodicity factor

    Returns
    -------
    I: ndarray
        Current matrix accounting for periodicities [q_pera,len(time)]
    """

    # Get lamination
    if is_stator:
        lam = self.parent.simu.machine.stator
    else:
        lam = self.parent.simu.machine.rotor

    if hasattr(lam, "winding") and lam.winding is not None:

        # Get the number of parallel circuit per phase of winding
        if hasattr(lam.winding, "Npcpp") and lam.winding.Npcpp is not None:
            Npcpp = lam.winding.Npcpp
        else:
            Npcpp = 1

        # Get current DataTime
        if is_stator:
            I_data = self.get_Is()
        else:
            I_data = self.Ir

        if phase is None:
            # Take all phases that are in the I_data Data object
            str_phase = "phase"
        else:
            str_phase = "phase" + str(phase)

        # Interpolate stator currents on input time vector
        I = (
            I_data.get_along(
                "time=axis_data",
                str_phase,
                axis_data={"time": time},
            )[I_data.symbol]
            / Npcpp
        )

        # Add time dimension if Is is calculated only for one time step
        if len(I.shape) == 1:
            I = I[:, None]

    else:
        I = None

    return I



# -*- coding: utf-8 -*-

from numpy import cos, exp, sin

from ....Classes.Arc1 import Arc1
from ....Classes.Segment import Segment
from ....Classes.SurfLine import SurfLine


def build_geometry(self, alpha=0, delta=0, is_simplified=False):
    """Compute the curve (Segment) needed to plot the Hole.
    The ending point of a curve is the starting point of the next curve in the
    list

    Parameters
    ----------
    self : HoleM53
        A HoleM53 object
    alpha : float
        Angle to rotate the slot (Default value = 0) [rad]
    delta : complex
        Complex to translate the slot (Default value = 0)
    is_simplified : bool
       True to avoid line superposition

    Returns
    -------
    surf_list: list
        List of Magnet Surface and Air Surface on the slot

    """
    if self.get_is_stator():  # check if the slot is on the stator
        st = "_Stator"
    else:
        st = "_Rotor"
    # Get all the points
    point_dict = self._comp_point_coordinate()
    Z1 = point_dict["Z1"]
    Z2 = point_dict["Z2"]
    Z3 = point_dict["Z3"]
    Z4 = point_dict["Z4"]
    Z5 = point_dict["Z5"]
    Z6 = point_dict["Z6"]
    Z7 = point_dict["Z7"]
    Z8 = point_dict["Z8"]
    Z9 = point_dict["Z9"]
    Z10 = point_dict["Z10"]
    Z11 = point_dict["Z11"]

    Z1s = point_dict["Z1s"]
    Z2s = point_dict["Z2s"]
    Z3s = point_dict["Z3s"]
    Z4s = point_dict["Z4s"]
    Z5s = point_dict["Z5s"]
    Z6s = point_dict["Z6s"]
    Z7s = point_dict["Z7s"]
    Z8s = point_dict["Z8s"]
    Z9s = point_dict["Z9s"]
    Z10s = point_dict["Z10s"]
    Z11s = point_dict["Z11s"]
    Rext = self.get_Rext()

    # Air surface with magnet_0
    curve_list = list()
    curve_list.append(Segment(Z1, Z2))
    curve_list.append(Segment(Z2, Z10))
    curve_list.append(Segment(Z10, Z11))
    curve_list.append(
        Arc1(begin=Z11, end=Z1, radius=-Rext + self.H1, is_trigo_direction=False)
    )
    point_ref = (Z1 + Z2 + Z10 + Z11) / 4
    S1 = SurfLine(line_list=curve_list, label="Hole" + st, point_ref=point_ref)

    # magnet_0 surface
    curve_list = list()
    if is_simplified:
        curve_list.append(Segment(Z5, Z9))
        curve_list.append(Segment(Z2, Z10))
    else:
        curve_list.append(Segment(Z3, Z4))
        curve_list.append(Segment(Z4, Z9))
        curve_list.append(Segment(Z9, Z10))
        curve_list.append(Segment(Z10, Z3))
    point_ref = (Z3 + Z4 + Z9 + Z10) / 4
    # Defining type of magnetization of the magnet
    if self.magnet_0:
        if self.magnet_0.type_magnetization == 0:
            type_mag = "_Radial"
        else:
            type_mag = "_Parallel"
    else:
        type_mag = "None"
    magnet_label = "HoleMagnet" + st + type_mag + "_N_R0_T0_S0"
    S2 = SurfLine(line_list=curve_list, label=magnet_label, point_ref=point_ref)

    # Air suface with magnet_0 and W1 > 0
    curve_list = list()
    if self.W2 > 0:
        curve_list.append(Segment(Z5, Z6))
    curve_list.append(Segment(Z6, Z7))
    curve_list.append(Segment(Z7, Z8))
    if self.W2 > 0:
        curve_list.append(Segment(Z8, Z9))
        curve_list.append(Segment(Z9, Z5))
        point_ref = (Z5 + Z6 + Z7 + Z8 + Z9) / 5
    else:
        curve_list.append(Segment(Z8, Z6))
        point_ref = (Z6 + Z7 + Z8) / 3
    S3 = SurfLine(line_list=curve_list, label="Hole" + st, point_ref=point_ref)

    # Air surface with magnet_1
    curve_list = list()
    curve_list.append(Segment(Z1s, Z2s))
    curve_list.append(Segment(Z2s, Z10s))
    curve_list.append(Segment(Z10s, Z11s))
    curve_list.append(Arc1(Z11s, Z1s, Rext - self.H1, is_trigo_direction=True))
    point_ref = (Z1s + Z2s + Z10s + Z11s) / 4
    S4 = SurfLine(line_list=curve_list, label="Hole" + st, point_ref=point_ref)

    # magnet_1 surface
    curve_list = list()
    if is_simplified:
        curve_list.append(Segment(Z5s, Z9s))
        curve_list.append(Segment(Z2s, Z10s))
    else:
        curve_list.append(Segment(Z3s, Z4s))
        curve_list.append(Segment(Z4s, Z9s))
        curve_list.append(Segment(Z9s, Z10s))
        curve_list.append(Segment(Z10s, Z3s))
    point_ref = (Z3s + Z4s + Z9s + Z10s) / 4
    # Defining type of magnetization of the magnet
    if self.magnet_1:
        if self.magnet_1.type_magnetization == 0:
            type_mag = "_Radial"
        else:
            type_mag = "_Parallel"
    else:
        type_mag = "None"
    magnet_label = "HoleMagnet" + st + type_mag + "_N_R0_T1_S0"
    S5 = SurfLine(line_list=curve_list, label=magnet_label, point_ref=point_ref)

    # Air suface with magnet_1 and W1 > 0
    curve_list = list()
    if self.W2 > 0:
        curve_list.append(Segment(Z5s, Z6s))
    curve_list.append(Segment(Z6s, Z7s))
    curve_list.append(Segment(Z7s, Z8s))
    if self.W2 > 0:
        curve_list.append(Segment(Z8s, Z9s))
        curve_list.append(Segment(Z9s, Z5s))
        point_ref = (Z5s + Z6s + Z7s + Z8s + Z9s) / 5
    else:
        curve_list.append(Segment(Z8s, Z6s))
        point_ref = (Z6s + Z7s + Z8s) / 3
    S6 = SurfLine(line_list=curve_list, label="Hole" + st, point_ref=point_ref)

    # Air with both magnet and W1 = 0
    curve_list = list()
    if self.W2 > 0:
        curve_list.append(Segment(Z5, Z6))
    curve_list.append(Segment(Z6, Z6s))
    if self.W2 > 0:
        curve_list.append(Segment(Z6s, Z5s))
    curve_list.append(Segment(Z5s, Z9s))
    if self.W2 > 0:
        curve_list.append(Segment(Z9s, Z8s))
        curve_list.append(Segment(Z8s, Z9))
    curve_list.append(Segment(Z9, Z5))
    point_ref = (Z6 + Z6s + Z8) / 3
    S7 = SurfLine(line_list=curve_list, label="Hole" + st, point_ref=point_ref)

    # first hole without magnet_0 and W1 > 0
    curve_list = list()
    curve_list.append(Segment(Z1, Z2))
    if self.H3 > 0:
        curve_list.append(Segment(Z2, Z3))
    curve_list.append(Segment(Z3, Z4))
    if self.H3 > 0:
        curve_list.append(Segment(Z4, Z5))
    if self.W2 > 0:
        curve_list.append(Segment(Z5, Z6))
    curve_list.append(Segment(Z6, Z7))
    curve_list.append(Segment(Z7, Z8))
    curve_list.append(Segment(Z8, Z11))
    curve_list.append(Arc1(Z11, Z1, -Rext + self.H1, is_trigo_direction=False))
    point_ref = (Z3 + Z4 + Z9 + Z10) / 4
    S8 = SurfLine(line_list=curve_list, label="Hole" + st, point_ref=point_ref)

    # second hole without magnet_1 and W1 > 0
    curve_list = list()
    curve_list.append(Segment(Z1s, Z2s))
    if self.H3 > 0:
        curve_list.append(Segment(Z2s, Z3s))
    curve_list.append(Segment(Z3s, Z4s))
    if self.H3 > 0:
        curve_list.append(Segment(Z4s, Z5s))
    if self.W2 > 0:
        curve_list.append(Segment(Z5s, Z6s))
    curve_list.append(Segment(Z6s, Z7s))
    curve_list.append(Segment(Z7s, Z8s))
    curve_list.append(Segment(Z8s, Z11s))
    curve_list.append(Arc1(Z11s, Z1s, -Rext + self.H1, is_trigo_direction=False))
    point_ref = (Z3s + Z4s + Z9s + Z10s) / 4
    S9 = SurfLine(line_list=curve_list, label="Hole" + st, point_ref=point_ref)

    # No magnet_1 and W1 = 0
    curve_list = list()
    curve_list.append(Segment(Z1s, Z2s))
    if self.H3 > 0:
        curve_list.append(Segment(Z2s, Z3s))
    curve_list.append(Segment(Z3s, Z4s))
    if self.H3 > 0:
        curve_list.append(Segment(Z4s, Z5s))
    if self.W2 > 0:
        curve_list.append(Segment(Z5s, Z6s))
    curve_list.append(Segment(Z6s, Z6))
    if self.W2 > 0:
        curve_list.append(Segment(Z6, Z5))
    curve_list.append(Segment(Z5, Z9))
    if self.W2 > 0:
        curve_list.append(Segment(Z9, Z8))
    curve_list.append(Segment(Z8s, Z11s))
    curve_list.append(Arc1(Z11s, Z1s, -Rext + self.H1, is_trigo_direction=False))
    point_ref = (Z3s + Z4s + Z9s + Z10s) / 4
    S10 = SurfLine(line_list=curve_list, label="Hole" + st, point_ref=point_ref)

    # No magnet_0 and W1 = 0
    curve_list = list()
    curve_list.append(Segment(Z1, Z2))
    if self.H3 > 0:
        curve_list.append(Segment(Z2, Z3))
    curve_list.append(Segment(Z3, Z4))
    if self.H3 > 0:
        curve_list.append(Segment(Z4, Z5))
    if self.W2 > 0:
        curve_list.append(Segment(Z5, Z6))
    curve_list.append(Segment(Z6, Z6s))
    if self.W2 > 0:
        curve_list.append(Segment(Z6s, Z5s))
    curve_list.append(Segment(Z5s, Z9s))
    if self.W2 > 0:
        curve_list.append(Segment(Z9s, Z8s))
    curve_list.append(Segment(Z8, Z11))
    curve_list.append(Arc1(Z11, Z1, -Rext + self.H1, is_trigo_direction=False))
    point_ref = (Z3 + Z4 + Z9 + Z10) / 4
    S11 = SurfLine(line_list=curve_list, label="Hole" + st, point_ref=point_ref)

    # No magnet and W1 = 0
    curve_list = list()
    curve_list.append(Arc1(Z1, Z11, Rext - self.H1, is_trigo_direction=True))
    curve_list.append(Segment(Z11, Z8))
    curve_list.append(Segment(Z8, Z11s))
    curve_list.append(Arc1(Z11s, Z1s, Rext - self.H1, is_trigo_direction=True))
    curve_list.append(Segment(Z1s, Z2s))
    if self.H3 > 0:
        curve_list.append(Segment(Z2s, Z3s))
    curve_list.append(Segment(Z3s, Z4s))
    if self.H3 > 0:
        curve_list.append(Segment(Z4s, Z5s))
    if self.W2 > 0:
        curve_list.append(Segment(Z5s, Z6s))
    curve_list.append(Segment(Z6s, Z6))
    if self.W2 > 0:
        curve_list.append(Segment(Z6, Z5))
    if self.H3 > 0:
        curve_list.append(Segment(Z5, Z4))
    curve_list.append(Segment(Z4, Z3))
    if self.H3 > 0:
        curve_list.append(Segment(Z3, Z2))
    curve_list.append(Segment(Z2, Z1))

    point_ref = (Z6 + Z8 + Z6s) / 3
    S12 = SurfLine(line_list=curve_list, label="Hole" + st, point_ref=point_ref)

    # Create the surface list by selecting the correct ones
    if self.magnet_0 and self.magnet_1 and self.W1 > 0:
        S1.label = S1.label + "_R0_T0_S0"  # Hole
        S3.label = S3.label + "_R0_T1_S0"  # Hole
        S6.label = S6.label + "_R0_T2_S0"  # Hole
        S4.label = S4.label + "_R0_T3_S0"  # Hole
        surf_list = [S1, S2, S3, S6, S5, S4]
    elif self.magnet_0 and self.magnet_1 and self.W1 == 0:
        S1.label = S1.label + "_R0_T0_S0"  # Hole
        S7.label = S7.label + "_R0_T1_S0"  # Hole
        S4.label = S4.label + "_R0_T2_S0"  # Hole
        surf_list = [S1, S2, S7, S5, S4]
    elif self.magnet_0 and not self.magnet_1 and self.W1 > 0:
        S1.label = S1.label + "_R0_T0_S0"  # Hole
        S3.label = S3.label + "_R0_T1_S0"  # Hole
        S9.label = S9.label + "_R0_T2_S0"  # Hole
        surf_list = [S1, S2, S3, S9]
    elif self.magnet_0 and not self.magnet_1 and self.W1 == 0:
        S1.label = S1.label + "_R0_T0_S0"  # Hole
        S10.label = S10.label + "_R0_T1_S0"  # Hole
        surf_list = [S1, S2, S10]
    elif not self.magnet_0 and self.magnet_1 and self.W1 > 0:
        S8.label = S8.label + "_R0_T0_S0"  # Hole
        S6.label = S6.label + "_R0_T1_S0"  # Hole
        S4.label = S4.label + "_R0_T2_S0"  # Hole
        surf_list = [S8, S6, S5, S4]
    elif not self.magnet_0 and self.magnet_1 and self.W1 == 0:
        S11.label = S11.label + "_R0_T0_S0"  # Hole
        S4.label = S4.label + "_R0_T2_S0"  # Hole
        surf_list = [S11, S5, S4]
    elif not self.magnet_0 and not self.magnet_1 and self.W1 > 0:
        S8.label = S8.label + "_R0_T0_S0"  # Hole
        S9.label = S9.label + "_R0_T1_S0"  # Hole
        surf_list = [S8, S9]
    elif not self.magnet_0 and not self.magnet_1 and self.W1 == 0:
        S12.label = S12.label + "_R0_T0_S0"  # Hole
        surf_list = [S12]

    # Apply the transformation
    for surf in surf_list:
        surf.rotate(alpha)
        surf.translate(delta)

    return surf_list



from logging import getLogger
from os.path import dirname, isfile

import matplotlib.pyplot as plt
from ezdxf import readfile
from numpy import angle as np_angle
from numpy import argmin, array
from PySide2.QtCore import QSize, Qt
from PySide2.QtGui import QIcon, QPixmap
from PySide2.QtWidgets import QComboBox, QDialog, QFileDialog, QMessageBox, QPushButton

from ...Classes.LamSlot import LamSlot
from ...Classes.Magnet import Magnet
from ...Classes.SlotUD import SlotUD
from ...definitions import config_dict
from ...GUI.Dxf.dxf_to_pyleecan_list import dxf_to_pyleecan_list
from ...GUI.Resources import pixmap_dict
from ...GUI.Tools.MPLCanvas import MPLCanvas2
from ...loggers import GUI_LOG_NAME
from .Ui_DXF_Slot import Ui_DXF_Slot

# Column index for table
TYPE_COL = 0
DEL_COL = 1
HL_COL = 2
WIND_COLOR = config_dict["PLOT"]["COLOR_DICT"]["BAR_COLOR"]


class DXF_Slot(Ui_DXF_Slot, QDialog):
    """Dialog to create SlotUD objects from DXF files"""

    def __init__(self, dxf_path=None, Zs=None, lam=None):
        """Initialize the Dialog

        Parameters
        ----------
        self : DXF_Slot
            a DXF_Slot object
        dxf_path : str
            Path to a dxf file to read
        """
        # Widget setup
        QDialog.__init__(self)
        self.setupUi(self)

        # Init properties
        self.line_list = list()  # List of line from DXF
        self.selected_list = list()  # List of currently selected lines
        self.lam = lam

        # Initialize the graph
        self.init_graph()

        # Not used yet
        self.in_coord_center.hide()
        self.lf_center_x.hide()
        self.lf_center_y.hide()
        self.lf_axe_angle.hide()
        self.in_axe_angle.hide()
        self.lf_center_x.setValue(0)
        self.lf_center_y.setValue(0)
        self.lf_scaling.validator().setBottom(0)
        self.lf_scaling.setValue(1)

        # Set default values
        if Zs is not None:
            self.si_Zs.setValue(Zs)

        # Setup Path selector for DXF files
        self.dxf_path = dxf_path
        self.w_path_selector.obj = self
        self.w_path_selector.param_name = "dxf_path"
        self.w_path_selector.verbose_name = "DXF File"
        self.w_path_selector.extension = "DXF file (*.dxf)"
        self.w_path_selector.set_path_txt(self.dxf_path)
        self.w_path_selector.update()

        # Load the DXF file if provided
        if self.dxf_path is not None and isfile(self.dxf_path):
            self.open_document()

        # Connect signals to slot
        self.w_path_selector.pathChanged.connect(self.open_document)
        self.b_save.pressed.connect(self.save)
        self.b_plot.pressed.connect(self.plot)
        self.b_reset.pressed.connect(self.update_graph)
        self.b_cancel.pressed.connect(self.remove_selection)

        # Display the GUI
        self.show()

    def open_document(self):
        """Open a new dxf in the viewer

        Parameters
        ----------
        self : DXF_Slot
            a DXF_Slot object
        """

        getLogger(GUI_LOG_NAME).debug("Reading dxf file: " + self.dxf_path)
        # Read the DXF file
        try:
            document = readfile(self.dxf_path)
            modelspace = document.modelspace()
            # Convert DXF to pyleecan objects
            self.line_list = dxf_to_pyleecan_list(modelspace)
            # Display
            self.selected_list = [False for line in self.line_list]
            self.update_graph()
        except Exception as e:
            QMessageBox().critical(
                self,
                self.tr("Error"),
                self.tr("Error while reading dxf file:\n" + str(e)),
            )

    def init_graph(self):
        """Initialize the viewer

        Parameters
        ----------
        self : DXF_Slot
            a DXF_Slot object
        """
        fig, axes = self.w_viewer.fig, self.w_viewer.axes
        axes.set_axis_off()

        # Setup interaction with graph
        def select_line(event):
            """Function to select/unselect the closest line from click"""
            X = event.xdata  # X position of the click
            Y = event.ydata  # Y position of the click
            # Get closer pyleecan object
            Z = X + 1j * Y
            min_dist = float("inf")
            closest_id = -1
            for ii, line in enumerate(self.line_list):
                line_dist = line.comp_distance(Z)
                if line_dist < min_dist:
                    closest_id = ii
                    min_dist = line_dist
            # Select/unselect line
            self.selected_list[closest_id] = not self.selected_list[closest_id]
            # Change line color
            point_list = array(self.line_list[closest_id].discretize(20))
            if self.selected_list[closest_id]:
                color = "r"
            else:
                color = "k"
            axes.plot(point_list.real, point_list.imag, color, zorder=2)
            self.w_viewer.draw()

        def zoom(event):
            """Function to zoom/unzoom according the mouse wheel"""

            base_scale = 0.3  # Scaling factor
            # get the current x and y limits
            ax = self.w_viewer.axes
            cur_xlim = ax.get_xlim()
            cur_ylim = ax.get_ylim()
            cur_xrange = (cur_xlim[1] - cur_xlim[0]) * 0.5
            cur_yrange = (cur_ylim[1] - cur_ylim[0]) * 0.5
            xdata = event.xdata  # get event x location
            ydata = event.ydata  # get event y location
            if event.button == "down":
                # deal with zoom in
                scale_factor = 1 / base_scale
            elif event.button == "up":
                # deal with zoom out
                scale_factor = base_scale
            else:
                # deal with something that should never happen
                scale_factor = 1
            # set new limits
            ax.set_xlim(
                [xdata - cur_xrange * scale_factor, xdata + cur_xrange * scale_factor]
            )
            ax.set_ylim(
                [ydata - cur_yrange * scale_factor, ydata + cur_yrange * scale_factor]
            )
            self.w_viewer.draw()  # force re-draw

        # Connect the function
        self.w_viewer.mpl_connect("button_press_event", select_line)
        self.w_viewer.mpl_connect("scroll_event", zoom)

        # Axis cleanup
        axes.axis("equal")
        axes.set_axis_off()

    def update_graph(self):
        """Clean and redraw all the lines in viewer

        Parameters
        ----------
        self : DXF_Slot
            a DXF_Slot object
        """
        fig, axes = self.w_viewer.fig, self.w_viewer.axes
        axes.clear()
        axes.set_axis_off()

        # Draw the lines in the correct color
        for ii, line in enumerate(self.line_list):
            point_list = array(line.discretize(20))
            if self.selected_list[ii]:
                color = "r"
            else:
                color = "k"
            axes.plot(point_list.real, point_list.imag, color, zorder=1)

        self.w_viewer.draw()

    def check_selection(self):
        """Check if every line in the selection are connected

        Parameters
        ----------
        self : DXF_Slot
            a DXF_Slot object

        Returns
        -------
        is_line : bool
            True if it forms a line
        """

        # Create list of begin and end point for all lines
        point_list = list()
        for ii, line in enumerate(self.line_list):
            if self.selected_list[ii]:
                point_list.append(line.get_begin())
                point_list.append(line.get_end())

        # Check with a tolerance if every point is twice in the list
        if len(point_list) == 0:
            return False

        # Number of point only 1 time in the list (begin and end)
        count_1 = 0
        for p1 in point_list:
            count = 0
            for p2 in point_list:
                if abs(p1 - p2) < 1e-9:
                    count += 1
            if count == 1:
                count_1 += 1
                if count_1 > 2:
                    return False
            elif count != 2:
                return False

        return True

    def remove_selection(self):
        # Remove selection
        self.selected_list = [False for line in self.line_list]
        self.update_graph()

    def get_slot(self):
        """Generate the SlotUD object corresponding to the selected lines

        Parameters
        ----------
        self : DXF_Slot
            a DXF_Slot object

        Returns
        -------
        sot : SlotUD
            User defined slot according to selected lines
        """

        if self.lf_scaling.value() == 0:  # Avoid error
            self.lf_scaling.setValue(1)

        # Get all the selected lines
        line_list = list()
        point_list = list()
        for ii, line in enumerate(self.line_list):
            if self.selected_list[ii]:
                line_list.append(line.copy())
                line_list[-1].scale(self.lf_scaling.value())
                point_list.append(line_list[-1].get_begin())
                point_list.append(line_list[-1].get_end())

        # Find begin point
        single_list = list()
        for p1 in point_list:
            count = 0
            for p2 in point_list:
                if abs(p1 - p2) < 1e-9:
                    count += 1
            if count == 1:
                single_list.append(p1)
        assert len(single_list) == 2
        Zbegin = single_list[argmin(np_angle(array(single_list)))]
        # Get begin line
        id_list = list()
        id_list.extend(
            [
                ii
                for ii, line in enumerate(line_list)
                if abs(line.get_begin() - Zbegin) < 1e-6
            ]
        )
        # Sort the lines (begin = end)
        curve_list = list()
        curve_list.append(line_list.pop(id_list[0]))
        while len(line_list) > 0:
            end = curve_list[-1].get_end()
            for ii in range(len(line_list)):
                if abs(line_list[ii].get_begin() - end) < 1e-9:
                    break
                if abs(line_list[ii].get_end() - end) < 1e-9:
                    line_list[ii].reverse()
                    break
            curve_list.append(line_list.pop(ii))

        # Create the Slot object
        slot = SlotUD(line_list=curve_list)
        slot.type_line_wind = self.c_type_line.currentIndex()
        begin_id = self.si_wind_begin_index.value()
        end_id = self.si_wind_end_index.value()
        if (
            begin_id < len(curve_list)
            and end_id < len(curve_list)
            and begin_id < end_id
        ):
            slot.wind_begin_index = begin_id
            slot.wind_end_index = end_id
        else:
            slot.wind_begin_index = None
            slot.wind_end_index = None

        # Rotation
        Z1 = curve_list[0].get_begin()
        Z2 = curve_list[-1].get_end()
        alpha = (np_angle(Z2) + np_angle(Z1)) / 2
        for line in curve_list:
            line.rotate(-1 * alpha)

        # Set metadata
        slot.Zs = self.si_Zs.value()

        return slot

    def plot(self):
        """Plot the current state of the hole

        Parameters
        ----------
        self : DXF_Slot
            a DXF_Slot object
        """
        if self.check_selection():
            slot = self.get_slot()
            # Lamination definition
            if self.lam is None:
                lam = LamSlot(slot=slot)
                Rbo = abs(slot.line_list[0].get_begin())
            else:
                lam = self.lam.copy()
                lam.slot = slot
            fig, (ax1, ax2) = plt.subplots(1, 2)
            slot.plot(fig=fig, ax=ax1)
            # Add the winding if defined
            if slot.wind_begin_index is not None:
                surf_wind = slot.get_surface_active()
                surf_wind.plot(fig=fig, ax=ax1, color=WIND_COLOR, is_show_fig=False)
            lam.plot(fig=fig, ax=ax2)

    def save(self):
        """Save the SlotUD object in a json file

        Parameters
        ----------
        self : DXF_Slot
            a DXF_Slot object
        """

        if self.check_selection():
            slot = self.get_slot()

            save_file_path = QFileDialog.getSaveFileName(
                self, self.tr("Save file"), dirname(self.dxf_path), "Json (*.json)"
            )[0]
            if save_file_path not in ["", ".json", None]:
                self.save_path = save_file_path
                slot.save(save_file_path)
                self.accept()



# -*- coding: utf-8 -*-


from PySide2.QtCore import Signal
from PySide2.QtWidgets import QMessageBox, QWidget


from .....GUI.Dialog.DMachineSetup.SPreview.Ui_SPreview import Ui_SPreview


class SPreview(Ui_SPreview, QWidget):
    """Step to define the winding conductor"""

    # Signal to DMachineSetup to know that the save popup is needed
    saveNeeded = Signal()
    # Information for DMachineSetup nav
    step_name = "Machine Summary"

    def __init__(self, machine, matlib, is_stator=False):
        """Initialize the GUI according to machine

        Parameters
        ----------
        self : SPreview
            A SPreview widget
        machine : Machine
            current machine to edit
        matlib : MatLib
            Material Library
        is_stator : bool
            To adapt the GUI to set either the stator or the rotor
        """

        # Build the interface according to the .ui file
        QWidget.__init__(self)
        self.setupUi(self)

        self.machine = machine
        # Update the preview
        self.tab_machine.update_tab(self.machine)

        self.machine.plot(
            fig=self.w_plot.fig,
            ax=self.w_plot.axes,
            sym=1,
            alpha=0,
            delta=0,
            is_show_fig=False,
        )
        self.w_plot.axes.set_axis_off()
        self.w_plot.axes.axis("equal")
        if self.w_plot.axes.get_legend() is not None:
            self.w_plot.axes.get_legend().remove()
        self.w_plot.draw()

    # def resizeEvent(self, event):
    #     W_main = self.width() - self.tab_machine.width()
    #     H_main = self.tab_machine.height()
    #     Size = max(min(W_main, H_main), 300)
    #     self.w_plot.resize(Size, Size)
    #     return super(SPreview, self).resizeEvent(event)



from numpy import sign


def comp_rot_dir(self):
    """Compute the rotation direction of the fundamental magnetic field induced by the winding
    WARNING: rot_dir = -1 to have positive rotor rotating direction, i.e. rotor position moves towards positive angle

    Parameters
    ----------
    self : LamSlotWind
        A LamSlotWind object

    Returns
    -------
    rot_dir : int
        -1 or +1
    """
    p = self.get_pole_pair_number()

    # Compute unit mmf
    MMF, _ = self.comp_mmf_unit(
        Nt=20 * p, Na=20 * p
    )  # 20 points per pole over time and space is enough to capture rotating direction of fundamental mmf

    # Extract fundamental from unit mmf
    results = MMF.get_harmonics(1, "freqs", "wavenumber")

    # Get frequency and wavenumber of fundamental
    f = results["freqs"][0]
    r = results["wavenumber"][0]

    # Rotating direction is the sign of the mechanical speed of the magnetic field fundamental, i.e frequency over wavenumber
    rot_dir = int(sign(f / r))

    return rot_dir



# -*- coding: utf-8 -*-

from PySide2.QtCore import Signal
from PySide2.QtGui import QPixmap
from PySide2.QtWidgets import QMessageBox, QWidget

from .....Classes.Frame import Frame
from .....GUI import gui_option
from .....GUI.Dialog.DMachineSetup.SMachineDimension.Ui_SMachineDimension import (
    Ui_SMachineDimension,
)
from .....GUI.Resources import pixmap_dict


class SMachineDimension(Ui_SMachineDimension, QWidget):
    """Step to setup the Machine dimension"""

    # Signal to DMachineSetup to know that the save popup is needed
    saveNeeded = Signal()
    # Information for the DMachineSetup nav
    step_name = "Machine Dimensions"

    def __init__(self, machine, matlib, is_stator=False):
        """Initialize the widget according to machine

        Parameters
        ----------
        self : SMachineDimension
            A SMachineDimension widget
        machine : Machine
            current machine to edit
        matlib : MatLib
            Material Library
        is_stator : bool
            To adapt the GUI to set either the stator or the rotor
        """

        # Build the interface according to the .ui file
        QWidget.__init__(self)
        self.setupUi(self)

        # Saving arguments
        self.machine = machine
        self.matlib = matlib
        self.is_stator = is_stator

        # Set FloatEdit unit
        self.lf_SRint.unit = "m"
        self.lf_SRext.unit = "m"
        self.lf_RRext.unit = "m"
        self.lf_RRint.unit = "m"
        self.lf_Wfra.unit = "m"
        self.lf_Lfra.unit = "m"
        # Set unit name (m ou mm)
        wid_list = [
            self.unit_SRint,
            self.unit_SRext,
            self.unit_RRext,
            self.unit_RRint,
            self.unit_Wfra,
            self.unit_Lfra,
        ]
        for wid in wid_list:
            wid.setText("[" + gui_option.unit.get_m_name() + "]")

        # Initialize the GUI with the current machine value
        self.lf_SRint.setValue(machine.stator.Rint)
        self.lf_SRext.setValue(machine.stator.Rext)
        self.lf_RRint.setValue(machine.rotor.Rint)
        self.lf_RRext.setValue(machine.rotor.Rext)

        self.set_airgap()  # Update out_airgap if possible

        # Set default materials
        self.w_mat_0.setText("Shaft Material")
        self.w_mat_0.def_mat = "M400-50A"
        self.w_mat_1.setText("Frame Material")
        self.w_mat_1.def_mat = "M400-50A"

        if (
            machine.frame is None
            or machine.frame.Rint is None
            or machine.frame.Rext is None
            or machine.frame.comp_height_eq() == 0
        ):
            self.g_frame.setChecked(False)
            self.lf_Wfra.clear()  # Empty spinbox
            self.lf_Lfra.clear()  # Empty spinbox
        else:
            self.g_frame.setChecked(True)
            Wfra = machine.frame.comp_height_eq()
            self.lf_Wfra.setText(format(gui_option.unit.get_m(Wfra), ".6g"))
            if machine.frame.Lfra is not None:
                self.lf_Lfra.setValue(machine.frame.Lfra)
            self.w_mat_1.update(self.machine.frame, "mat_type", self.matlib)

        # Adapt the GUI to the topology of the machine
        if not machine.rotor.is_internal:  # External Rotor
            self.img_machine.setPixmap(QPixmap(pixmap_dict["Dim_Ext_Rotor"]))
            self.g_shaft.hide()
            self.g_frame.hide()
        elif (
            machine.shaft is None
            or machine.shaft.Drsh is None
            or machine.shaft.Drsh == 0
        ):
            # Internal Rotor without shaft
            self.g_shaft.show()
            self.g_frame.show()
            self.img_machine.setPixmap(QPixmap(pixmap_dict["Dim_In_Rotor_No_Shaft"]))
            self.g_shaft.setChecked(False)
            self.out_Drsh.hide()
            # If there is no shaft, the rotor doesn't have internal radius
            self.lf_RRint.setValue(0)
            self.lf_RRint.setEnabled(False)
            self.machine.rotor.Rint = 0
        else:  # Internal Rotor with shaft
            self.g_shaft.show()
            self.g_frame.show()
            self.img_machine.setPixmap(QPixmap(pixmap_dict["Dim_In_Rotor_Shaft"]))
            self.g_shaft.setChecked(True)
            if machine.shaft.Drsh is not None:
                self.out_Drsh.setText(
                    self.tr("Drsh = 2*Rotor.Rint = ")
                    + str(2000 * self.machine.rotor.Rint)
                    + " mm"
                )

            self.w_mat_0.update(self.machine.shaft, "mat_type", self.matlib)

        # Connect the widget
        self.lf_SRint.editingFinished.connect(self.set_stator_Rint)
        self.lf_SRext.editingFinished.connect(self.set_stator_Rext)
        self.lf_RRint.editingFinished.connect(self.set_rotor_Rint)
        self.lf_RRext.editingFinished.connect(self.set_rotor_Rext)
        self.lf_Wfra.editingFinished.connect(self.set_Wfra)
        self.lf_Lfra.editingFinished.connect(self.set_Lfra)

        self.g_shaft.toggled.connect(self.set_Drsh)
        self.g_frame.toggled.connect(self.clear_frame)

        self.w_mat_0.saveNeeded.connect(self.emit_save)
        self.w_mat_1.saveNeeded.connect(self.emit_save)

    def set_stator_Rint(self):
        """Signal to update the value of stator.Rint according to the line edit

        Parameters
        ----------
        self : SMachineDimension
            A SMachineDimension object
        """
        self.machine.stator.Rint = self.lf_SRint.value()
        self.set_airgap()  # Update out_airgap if possible
        # Notify the machine GUI that the machine has changed
        self.saveNeeded.emit()

    def set_stator_Rext(self):
        """Signal to update the value of stator.Rext according to the line edit

        Parameters
        ----------
        self : SMachineDimension
            A SMachineDimension object
        """
        if self.machine.stator.Rext != self.lf_SRext.value():
            Rext = self.lf_SRext.value()
            self.machine.stator.Rext = Rext
            if self.machine.frame is not None:
                try:  # Fail if lf_Wfra empty
                    self.machine.frame.Rext = Rext + self.lf_Wfra.value()
                    self.machine.frame.Rint = Rext
                except TypeError:  # Wfra is None
                    self.machine.frame.Rext = Rext
                    self.machine.frame.Rint = Rext
            self.set_airgap()  # Update out_airgap if possible
            # Notify the machine GUI that the machine has changed
            self.saveNeeded.emit()

    def set_rotor_Rint(self):
        """Signal to update the value of rotor.Rint according to the line edit

        Parameters
        ----------
        self : SMachineDimension
            A SMachineDimension object
        """
        self.machine.rotor.Rint = self.lf_RRint.value()
        if self.machine.rotor.is_internal:  # Update out_Drsh if needed
            self.set_Drsh(self.g_shaft.isChecked())
        self.set_airgap()  # Update out_airgap if possible
        # Notify the machine GUI that the machine has changed
        self.saveNeeded.emit()

    def set_rotor_Rext(self):
        """Signal to update the value of rotor.Rext according to the line edit

        Parameters
        ----------
        self : SMachineDimension
            A SMachineDimension object
        """
        self.machine.rotor.Rext = self.lf_RRext.value()
        self.set_airgap()  # Update out_airgap if possible
        # Notify the machine GUI that the machine has changed
        self.saveNeeded.emit()

    def set_airgap(self):
        """Signal to update the value of airgap according to the line edit

        Parameters
        ----------
        self : SMachineDimension
            A SMachineDimension object
        """
        # For readibility
        rotor = self.machine.rotor
        stator = self.machine.stator
        gap_txt = self.tr("gap = ")
        # Airgap definition change accoding to Topology
        if rotor.is_internal:
            # Update only if the needed parameters are set
            if rotor.Rext is not None and stator.Rint is not None:
                gap = stator.Rint - rotor.Rext
                airgap = format(gap * 1000, ".6g")
                self.out_airgap.setText(gap_txt + airgap + " mm")
            else:
                self.out_airgap.setText(gap_txt + "?")
        else:
            # Update only if the needed parameters are set
            if rotor.Rint is not None and stator.Rext is not None:
                airgap = format((rotor.Rint - stator.Rext) * 1000, ".6g")
                self.out_airgap.setText(gap_txt + airgap + " mm")
            else:
                self.out_airgap.setText(gap_txt + "?")

    def set_Wfra(self):
        """Signal to update the value of Wfra according to the spinbox

        Parameters
        ----------
        self : SMachineDimension
            A SMachineDimension object
        """
        if self.machine.stator.Rext is not None:
            Wfra = self.lf_Wfra.value()
            self.machine.frame.Rint = self.machine.stator.Rext
            self.machine.frame.Rext = self.machine.stator.Rext + Wfra
        # Notify the machine GUI that the machine has changed
        self.saveNeeded.emit()

    def set_Lfra(self):
        """Signal to update the value of Lfra according to the line edit

        Parameters
        ----------
        self : SMachineDimension
            A SMachineDimension object
        """
        self.machine.frame.Lfra = self.lf_Lfra.value()
        # Notify the machine GUI that the machine has changed
        self.saveNeeded.emit()

    def clear_frame(self, is_checked):
        """Signal to remove the frame if the checkbox is unchecked

        Parameters
        ----------
        self : SMachineDimension
            A SMachineDimension object
        is_checked : bool
            State of the g_frame checkbox
        """
        if is_checked:
            self.machine.frame = Frame()
            self.machine.frame._set_None()
            if self.machine.stator.Rext is not None:
                self.machine.frame.Rint = self.machine.stator.Rext
                self.machine.frame.Rext = self.machine.stator.Rext
            else:
                self.machine.frame.Rint = None
                self.machine.frame.Rext = None
            self.w_mat_1.update(self.machine.frame, "mat_type", self.matlib)
            self.lf_Wfra.clear()
            self.lf_Lfra.clear()
        else:
            self.machine.frame = None
        # Notify the machine GUI that the machine has changed
        self.saveNeeded.emit()

    def set_Drsh(self, is_checked):
        """Signal to set Drsh according to the page context

        Parameters
        ----------
        self : SMachineDimension
            A SMachineDimension object
        is_checked : bool
            State of g_shaft
        """

        if is_checked:  # If there is a shaft
            # Set the corresponding image
            self.img_machine.setPixmap(QPixmap(pixmap_dict["Dim_In_Rotor_Shaft"]))
            self.w_mat_0.update(self.machine.shaft, "mat_type", self.matlib)
            # Set Drsh if machine.rotor.Rint is set
            if self.machine.rotor.Rint is not None:
                self.out_Drsh.setText(
                    self.tr("Drsh = 2*Rotor.Rint = ")
                    + str(2000 * self.machine.rotor.Rint)
                    + " mm"
                )
                self.machine.shaft.Drsh = self.machine.rotor.Rint * 2
            else:
                self.out_Drsh.setText(self.tr("Drsh = 2*Rotor.Rint = "))
                self.machine.shaft.Drsh = None

            # machine.rotor.Rint editable only if there is a shaft
            self.lf_RRint.setEnabled(True)

        else:  # If there is no shaft
            # Set the corresponding image
            self.img_machine.setPixmap(QPixmap(pixmap_dict["Dim_In_Rotor_No_Shaft"]))

            self.machine.shaft.Drsh = 0
            self.machine.rotor.Rint = 0
            self.lf_RRint.setValue(0)

            # machine.rotor.Rint editable only if there is a shaft
            self.lf_RRint.setEnabled(False)
        # Notify the machine GUI that the machine has changed
        self.saveNeeded.emit()

    @staticmethod
    def check(machine):
        """Check that the current machine have all the needed field set

        Parameters
        ----------
        machine : Machine
            Machine to check

        Returns
        -------
        error : str
            Error message (return None if no error)

        """
        try:
            # Check that everything is set
            if machine.stator.Rint is None:
                return "You must set Stator.Rint !"
            if machine.stator.Rext is None:
                return "You must set Stator.Rext !"
            if machine.rotor.Rint is None:
                return "You must set Rotor.Rint !"
            if machine.rotor.Rext is None:
                return "You must set Rotor.Rext !"

            # Check that everything is set right
            if machine.stator.Rext <= machine.stator.Rint:
                return "The Stator can't have an internal radius greater than the external one !"
            if machine.rotor.Rext <= machine.rotor.Rint:
                return "The Rotor can't have an internal radius greater than the external one !"
            if machine.rotor.is_internal and machine.stator.Rint <= machine.rotor.Rext:
                return (
                    "For inner rotor machine, you must have: Rotor.Rext < Stator.Rint !"
                )
            if (
                not machine.rotor.is_internal
                and machine.stator.Rext >= machine.rotor.Rint
            ):
                return "For external rotor machine, you must have: Stator.Rext < Rotor.Rint !"
        except Exception as e:
            return str(e)

    def check_gui(self):
        """Check that the widget are set right according to the current machine

        Parameters
        ----------
        self : SMachineDimension
            A SMachineDimension object
        """
        machine = self.machine

        if machine.rotor.Rint is None and self.g_shaft.isChecked():
            return self.tr("You must set Rotor.Rint !")
        if machine.rotor.Rint is None and not self.g_shaft.isChecked():
            machine.rotor.Rint = 0
        if self.g_frame.isChecked() and machine.frame.Rint is None:
            return self.tr("You must set Wfra or unchecked Frame !")
        if not self.g_frame.isChecked():
            self.machine.frame = None

    def emit_save(self):
        """Send a saveNeeded signal to the DMachineSetup"""
        self.saveNeeded.emit()



# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/GUI_Option/Unit.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/GUI_Option/Unit
"""

from os import linesep
from sys import getsizeof
from logging import getLogger
from ._check import check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from ._frozen import FrozenClass

# Import all class method
# Try/catch to remove unnecessary dependencies in unused method
try:
    from ..Methods.GUI_Option.Unit.get_m import get_m
except ImportError as error:
    get_m = error

try:
    from ..Methods.GUI_Option.Unit.get_m2 import get_m2
except ImportError as error:
    get_m2 = error

try:
    from ..Methods.GUI_Option.Unit.get_m_name import get_m_name
except ImportError as error:
    get_m_name = error

try:
    from ..Methods.GUI_Option.Unit.get_m2_name import get_m2_name
except ImportError as error:
    get_m2_name = error

try:
    from ..Methods.GUI_Option.Unit.set_m import set_m
except ImportError as error:
    set_m = error

try:
    from ..Methods.GUI_Option.Unit.set_m2 import set_m2
except ImportError as error:
    set_m2 = error


from ._check import InitUnKnowClassError


class Unit(FrozenClass):

    VERSION = 1

    # Check ImportError to remove unnecessary dependencies in unused method
    # cf Methods.GUI_Option.Unit.get_m
    if isinstance(get_m, ImportError):
        get_m = property(
            fget=lambda x: raise_(
                ImportError("Can't use Unit method get_m: " + str(get_m))
            )
        )
    else:
        get_m = get_m
    # cf Methods.GUI_Option.Unit.get_m2
    if isinstance(get_m2, ImportError):
        get_m2 = property(
            fget=lambda x: raise_(
                ImportError("Can't use Unit method get_m2: " + str(get_m2))
            )
        )
    else:
        get_m2 = get_m2
    # cf Methods.GUI_Option.Unit.get_m_name
    if isinstance(get_m_name, ImportError):
        get_m_name = property(
            fget=lambda x: raise_(
                ImportError("Can't use Unit method get_m_name: " + str(get_m_name))
            )
        )
    else:
        get_m_name = get_m_name
    # cf Methods.GUI_Option.Unit.get_m2_name
    if isinstance(get_m2_name, ImportError):
        get_m2_name = property(
            fget=lambda x: raise_(
                ImportError("Can't use Unit method get_m2_name: " + str(get_m2_name))
            )
        )
    else:
        get_m2_name = get_m2_name
    # cf Methods.GUI_Option.Unit.set_m
    if isinstance(set_m, ImportError):
        set_m = property(
            fget=lambda x: raise_(
                ImportError("Can't use Unit method set_m: " + str(set_m))
            )
        )
    else:
        set_m = set_m
    # cf Methods.GUI_Option.Unit.set_m2
    if isinstance(set_m2, ImportError):
        set_m2 = property(
            fget=lambda x: raise_(
                ImportError("Can't use Unit method set_m2: " + str(set_m2))
            )
        )
    else:
        set_m2 = set_m2
    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(self, unit_m=0, unit_rad=0, unit_m2=0, init_dict=None, init_str=None):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "unit_m" in list(init_dict.keys()):
                unit_m = init_dict["unit_m"]
            if "unit_rad" in list(init_dict.keys()):
                unit_rad = init_dict["unit_rad"]
            if "unit_m2" in list(init_dict.keys()):
                unit_m2 = init_dict["unit_m2"]
        # Set the properties (value check and convertion are done in setter)
        self.parent = None
        self.unit_m = unit_m
        self.unit_rad = unit_rad
        self.unit_m2 = unit_m2

        # The class is frozen, for now it's impossible to add new properties
        self._freeze()

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        Unit_str = ""
        if self.parent is None:
            Unit_str += "parent = None " + linesep
        else:
            Unit_str += "parent = " + str(type(self.parent)) + " object" + linesep
        Unit_str += "unit_m = " + str(self.unit_m) + linesep
        Unit_str += "unit_rad = " + str(self.unit_rad) + linesep
        Unit_str += "unit_m2 = " + str(self.unit_m2) + linesep
        return Unit_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False
        if other.unit_m != self.unit_m:
            return False
        if other.unit_rad != self.unit_rad:
            return False
        if other.unit_m2 != self.unit_m2:
            return False
        return True

    def compare(self, other, name="self"):
        """Compare two objects and return list of differences"""

        if type(other) != type(self):
            return ["type(" + name + ")"]
        diff_list = list()
        if other._unit_m != self._unit_m:
            diff_list.append(name + ".unit_m")
        if other._unit_rad != self._unit_rad:
            diff_list.append(name + ".unit_rad")
        if other._unit_m2 != self._unit_m2:
            diff_list.append(name + ".unit_m2")
        return diff_list

    def __sizeof__(self):
        """Return the size in memory of the object (including all subobject)"""

        S = 0  # Full size of the object
        S += getsizeof(self.unit_m)
        S += getsizeof(self.unit_rad)
        S += getsizeof(self.unit_m2)
        return S

    def as_dict(self, **kwargs):
        """
        Convert this object in a json serializable dict (can be use in __init__).
        Optional keyword input parameter is for internal use only
        and may prevent json serializability.
        """

        Unit_dict = dict()
        Unit_dict["unit_m"] = self.unit_m
        Unit_dict["unit_rad"] = self.unit_rad
        Unit_dict["unit_m2"] = self.unit_m2
        # The class name is added to the dict for deserialisation purpose
        Unit_dict["__class__"] = "Unit"
        return Unit_dict

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        self.unit_m = None
        self.unit_rad = None
        self.unit_m2 = None

    def _get_unit_m(self):
        """getter of unit_m"""
        return self._unit_m

    def _set_unit_m(self, value):
        """setter of unit_m"""
        check_var("unit_m", value, "int", Vmin=0, Vmax=1)
        self._unit_m = value

    unit_m = property(
        fget=_get_unit_m,
        fset=_set_unit_m,
        doc=u"""0: use m, 1: use mm

        :Type: int
        :min: 0
        :max: 1
        """,
    )

    def _get_unit_rad(self):
        """getter of unit_rad"""
        return self._unit_rad

    def _set_unit_rad(self, value):
        """setter of unit_rad"""
        check_var("unit_rad", value, "int", Vmin=0, Vmax=1)
        self._unit_rad = value

    unit_rad = property(
        fget=_get_unit_rad,
        fset=_set_unit_rad,
        doc=u"""0: use rad, 1: use deg

        :Type: int
        :min: 0
        :max: 1
        """,
    )

    def _get_unit_m2(self):
        """getter of unit_m2"""
        return self._unit_m2

    def _set_unit_m2(self, value):
        """setter of unit_m2"""
        check_var("unit_m2", value, "int", Vmin=0, Vmax=1)
        self._unit_m2 = value

    unit_m2 = property(
        fget=_get_unit_m2,
        fset=_set_unit_m2,
        doc=u"""0: use m^2, 1: use mm^2

        :Type: int
        :min: 0
        :max: 1
        """,
    )



# -*- coding: utf-8 -*-

from numpy import mod, pi


def comp_cond_function(alpha_cond, W0, alpha_rad):
    """Single conductor winding function for magnetomotive force calculation

    Parameters
    ----------
    alpha_cond : float
        angular position of the conductor [rad]
    W0 : float
        slot opening angular width (for linear rise of the mmf) [rad]
    alpha_rad : ndarray
        Position vector to compute the conductor winding function

    Returns
    -------
    wf: ndarray
        The conductor winding function

    """

    alpha_rad = mod(alpha_rad + pi - alpha_cond, 2 * pi)

    thi = 0
    thf = 2 * pi
    x1 = pi - W0 / 2.0
    x2 = pi + W0 / 2.0

    A = (alpha_rad < x1) * 1
    B = (alpha_rad > x2) * 1

    C = 1 - A - B
    return (
        C * (alpha_rad - pi) / (W0 / 2.0)
        + B * ((alpha_rad - thf) / (x2 - thf))
        - A * ((alpha_rad - thi) / (x1 - thi))
    )



from numpy import pi
from SciDataTool import Data1D, DataLinspace
from ....Methods.Simulation.Input import InputError


def comp_axes(
    self,
    axes_values,
    machine=None,
    N0=None,
    per_a=1,
    is_antiper_a=False,
    per_t=1,
    is_antiper_t=False,
):
    """Compute simulation axes, i.e. space DataObject including (anti)-periodicity
    and time DataObject including (anti)-periodicity and accounting for rotating speed
    and number of revolutions -> overrides Input comp_axes method

    Parameters
    ----------
    self : InputFlux
        an InputFlux object
    axes_values : {ndarray}
        dict of axe values
    machine : Machine
        a Machine object
    N0 : float
        rotating speed [rpm]
    per_a : int
        angle periodicity
    is_antiper_a : bool
        if the angle axis is antiperiodic
    per_t : int
        time periodicity
    is_antiper_t : bool
        if the time axis is antiperiodic


    Returns
    -------
    axes_dict : {Data}
        dict of Data objects for each axis

    """

    norm_time = {}
    norm_angle = {}

    if machine is not None:
        # Get machine pole pair number
        p = machine.get_pole_pair_number()

        # Get electrical fundamental frequency
        f_elec = self.comp_felec()

        # Airgap radius
        Rag = machine.comp_Rgap_mec()

        # Setup normalizations for time and angle axes
        norm_time["elec_order"] = f_elec
        norm_time["mech_order"] = f_elec / p
        if N0 is not None:
            norm_time["angle_rotor"] = 1 / (360 * N0 / 60)

        norm_angle["space_order"] = p
        norm_angle["distance"] = 1 / Rag

    sym_t = {}
    if is_antiper_t:
        sym_t["antiperiod"] = per_t
    else:
        sym_t["period"] = per_t

    if self.time is not None:
        Time = Data1D(
            name="time",
            unit="s",
            values=self.time.get_data(),
            normalizations=norm_time,
            symmetries=sym_t,
        )
    elif "time" in axes_values:
        Time = Data1D(
            name="time",
            unit="s",
            values=axes_values["time"],
            normalizations=norm_time,
            symmetries=sym_t,
        )
    elif N0 is None:
        raise InputError("ERROR: time and N0 can't be both None")
    else:
        # Create time axis as a DataLinspace
        Time = DataLinspace(
            name="time",
            unit="s",
            initial=0,
            final=60 / N0 * self.Nrev,
            number=self.Nt_tot,
            include_endpoint=False,
            normalizations=norm_time,
            symmetries=sym_t,
        )

    sym_a = {}
    if is_antiper_a:
        sym_a["antiperiod"] = per_a
    else:
        sym_a["period"] = per_a

    if self.angle is not None:
        Angle = Data1D(
            name="angle",
            unit="rad",
            values=self.angle.get_data(),
            normalizations=norm_angle,
            symmetries=sym_a,
        )
    elif "angle" in axes_values:
        Angle = Data1D(
            name="angle",
            unit="rad",
            values=axes_values["angle"],
            normalizations=norm_angle,
            symmetries=sym_a,
        )
    else:
        # Create angle axis as a DataLinspace
        Angle = DataLinspace(
            name="angle",
            unit="rad",
            initial=0,
            final=2 * pi,
            number=self.Na_tot,
            include_endpoint=False,
            normalizations=norm_angle,
            symmetries=sym_a,
        )

    # Store in axes_dict
    axes_dict = {"Time": Time, "Angle": Angle}

    return axes_dict



from sys import getsizeof
from ....Functions.Load.import_class import import_class


def print_memory(self, tab_level=0, detail_level=1, is_print=True):
    """Print an estimation of the memory usage for each property of the object

    Parameters
    ----------
    self : Output
        An Output object
    tab_level : int
        Number of tab ("    ") to add before each line
    detail_level : int
        Select how many data to display (0: only object size, 1: object + properties, 2:object + detailed properties)
    is_print: bool
        True print, else return str

    Returns
    -------
    mem_str: str
        if is_print == False, return the string that descibe the memory usage of the object
    """

    tab = ""
    for _ in range(tab_level):
        tab += "    "

    Output = import_class("pyleecan.Classes", "Output")
    if type(self) is Output:
        mem_str = tab + "Total Output size: " + str(getsizeof(self)) + " o\n"
    else:  # XOutput (print only the Output part of the size)
        mem_str = tab + "Total Output size: " + str(Output.__sizeof__(self)) + " o\n"
    if detail_level == 0:
        print(mem_str)
        return
    mem_str += tab + "path_result: " + str(getsizeof(self.path_result)) + " o\n"
    mem_str += tab + "logger_name: " + str(getsizeof(self.logger_name)) + " o\n"

    mem_str += tab + "simu: " + str(getsizeof(self.simu)) + " o\n"
    if detail_level > 1:
        mem_str += detail_prop(self, "simu", tab_level=tab_level + 1)

    mem_str += tab + "geo: " + str(getsizeof(self.geo)) + " o\n"
    if detail_level > 1:
        mem_str += detail_prop(self, "geo", tab_level=tab_level + 1)

    mem_str += tab + "elec: " + str(getsizeof(self.elec)) + " o\n"
    if detail_level > 1:
        mem_str += detail_prop(self, "elec", tab_level=tab_level + 1)

    mem_str += tab + "mag: " + str(getsizeof(self.mag)) + " o\n"
    if detail_level > 1:
        mem_str += detail_prop(self, "mag", tab_level=tab_level + 1)

    mem_str += tab + "force: " + str(getsizeof(self.force)) + " o\n"
    if detail_level > 1:
        mem_str += detail_prop(self, "force", tab_level=tab_level + 1)

    mem_str += tab + "struct: " + str(getsizeof(self.struct)) + " o\n"
    if detail_level > 1:
        mem_str += detail_prop(self, "struct", tab_level=tab_level + 1)

    mem_str += tab + "post: " + str(getsizeof(self.post)) + " o\n"

    if is_print:
        print(mem_str)
    else:
        return mem_str


def detail_prop(self, prop_name, tab_level=1):
    """Detail the content of the property"""

    tab = ""
    for _ in range(tab_level):
        tab += "    "

    mem_str = ""
    prop = getattr(self, prop_name)
    for attr in dir(prop):
        if (
            # Not method
            not callable(getattr(prop, attr))
            # Not private properties
            and not attr.startswith("_")
            # Not following properties
            and attr not in ["VERSION", "logger_name", "parent"]
        ):
            mem_str += (
                tab
                + prop_name
                + "."
                + attr
                + ": "
                + str(getsizeof(getattr(prop, attr)))
                + " o\n"
            )
    return mem_str



# -*- coding: utf-8 -*-

from numpy import arcsin


def comp_angle_bottom(self):
    """Compute the slot bottom angle of the Slot

    Parameters
    ----------
    self : Slot19
        A Slot19 object

    Returns
    -------
    alpha: float
        slot bottom angle [rad]

    """

    Rbo = self.get_Rbo()

    if self.Wx_is_rad:
        return self.W1
    else:
        if self.is_outwards():
            alpha = 2 * float(arcsin(self.W1 / (2 * Rbo + 2 * self.H0)))
        else:  # inward slot
            alpha = 2 * float(arcsin(self.W1 / (2 * Rbo - 2 * self.H0)))

    return alpha



# -*- coding: utf-8 -*-


def comp_radius_mid_active(self):
    """Compute the radius at the middle of the active part of the slot

    Parameters
    ----------
    self : Slot
        A Slot object

    Returns
    -------
    Rmw: float
        Mid active radius [m]

    """

    Rbo = self.get_Rbo()
    Hslot = self.comp_height()
    Hwind = self.comp_height_active()
    if self.is_outwards():
        return Rbo + Hslot - Hwind / 2
    else:
        return Rbo - Hslot + Hwind / 2



from PySide2.QtCore import QSize
from PySide2.QtWidgets import QGroupBox, QLabel, QVBoxLayout, QWidget

from ......GUI import gui_option


class WWSlotOut(QGroupBox):
    """Setup of QGroupBox for output for Winding Slot"""

    def __init__(self, parent=None):
        """Initialize the widget"""

        QGroupBox.__init__(self, parent)
        # Set main widget
        self.setTitle(self.tr("Output"))
        self.setMinimumSize(QSize(200, 0))
        self.setObjectName("g_output")
        self.layout = QVBoxLayout(self)
        self.layout.setObjectName("layout")
        # The widget is composed of several QLabel in a vertical layout
        self.out_Wlam = QLabel(self)
        self.out_Wlam.setObjectName("out_Wlam")
        self.layout.addWidget(self.out_Wlam)

        self.out_slot_height = QLabel(self)
        self.out_slot_height.setObjectName("out_slot_height")
        self.layout.addWidget(self.out_slot_height)

        self.out_yoke_height = QLabel(self)
        self.out_yoke_height.setObjectName("out_yoke_height")
        self.layout.addWidget(self.out_yoke_height)

        self.out_wind_surface = QLabel(self)
        self.out_wind_surface.setObjectName("out_wind_surface")
        self.layout.addWidget(self.out_wind_surface)

        self.out_tot_surface = QLabel(self)
        self.out_tot_surface.setObjectName("out_tot_surface")
        self.layout.addWidget(self.out_tot_surface)

        self.out_op_angle = QLabel(self)
        self.out_op_angle.setObjectName("out_op_angle")
        self.layout.addWidget(self.out_op_angle)

        # self.layout.addWidget(self)

    def comp_output(self):
        """Update the Output with the computed values

        Parameters
        ----------
        self : WWSlotOut
            A WWSlotOut object
        """

        if self.parent() is not None and hasattr(self.parent(), "lamination"):
            parent = self.parent()
            lam = self.parent().lamination
        else:
            parent = self.parent().parent()
            lam = self.parent().parent().lamination
        if lam.is_stator:
            lam_name = "Stator"
        else:
            lam_name = "Rotor"

        WS_txt = self.tr("Active surface: ")
        TS_txt = self.tr("Slot surface: ")
        AO_txt = self.tr("Opening angle: ")
        SH_txt = self.tr("Slot height: ")
        YH_txt = self.tr("Yoke height: ")

        Wlam = format(gui_option.unit.get_m(lam.Rext - lam.Rint), ".4g")
        self.out_Wlam.setText(
            self.tr(lam_name + " width: ")
            + Wlam
            + " ["
            + gui_option.unit.get_m_name()
            + "]"
        )
        if parent.check(lam) is None:
            # Compute all the needed output as string
            w_surf = format(
                gui_option.unit.get_m2(lam.slot.comp_surface_active()), ".4g"
            )
            tot_surf = format(gui_option.unit.get_m2(lam.slot.comp_surface()), ".4g")
            op_angle = "%.4g" % lam.slot.comp_angle_opening()
            slot_height = format(gui_option.unit.get_m(lam.slot.comp_height()), ".4g")
            yoke_height = format(gui_option.unit.get_m(lam.comp_height_yoke()), ".4g")

            # Update the GUI to display the Output
            self.out_wind_surface.setText(
                WS_txt + w_surf + " [" + gui_option.unit.get_m2_name() + "]"
            )
            self.out_tot_surface.setText(
                TS_txt + tot_surf + " [" + gui_option.unit.get_m2_name() + "]"
            )
            self.out_op_angle.setText(AO_txt + op_angle + u" [rad]")
            self.out_slot_height.setText(
                SH_txt + slot_height + " [" + gui_option.unit.get_m_name() + "]"
            )
            self.out_yoke_height.setText(
                YH_txt + yoke_height + " [" + gui_option.unit.get_m_name() + "]"
            )
        else:
            # We can't compute the output => We erase the previous version
            # (that way the user know that something is wrong)
            self.out_wind_surface.setText(WS_txt + "?")
            self.out_tot_surface.setText(TS_txt + "?")
            self.out_op_angle.setText(AO_txt + "?")
            self.out_slot_height.setText(SH_txt + "?")
            self.out_yoke_height.setText(YH_txt + "?")



# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/Machine/Magnet.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/Machine/Magnet
"""

from os import linesep
from sys import getsizeof
from logging import getLogger
from ._check import check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from ._frozen import FrozenClass

from ._check import InitUnKnowClassError
from .Material import Material


class Magnet(FrozenClass):
    """Magnet class"""

    VERSION = 1

    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(
        self,
        mat_type=-1,
        type_magnetization=0,
        Lmag=0.95,
        init_dict=None,
        init_str=None,
    ):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "mat_type" in list(init_dict.keys()):
                mat_type = init_dict["mat_type"]
            if "type_magnetization" in list(init_dict.keys()):
                type_magnetization = init_dict["type_magnetization"]
            if "Lmag" in list(init_dict.keys()):
                Lmag = init_dict["Lmag"]
        # Set the properties (value check and convertion are done in setter)
        self.parent = None
        self.mat_type = mat_type
        self.type_magnetization = type_magnetization
        self.Lmag = Lmag

        # The class is frozen, for now it's impossible to add new properties
        self._freeze()

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        Magnet_str = ""
        if self.parent is None:
            Magnet_str += "parent = None " + linesep
        else:
            Magnet_str += "parent = " + str(type(self.parent)) + " object" + linesep
        if self.mat_type is not None:
            tmp = self.mat_type.__str__().replace(linesep, linesep + "\t").rstrip("\t")
            Magnet_str += "mat_type = " + tmp
        else:
            Magnet_str += "mat_type = None" + linesep + linesep
        Magnet_str += "type_magnetization = " + str(self.type_magnetization) + linesep
        Magnet_str += "Lmag = " + str(self.Lmag) + linesep
        return Magnet_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False
        if other.mat_type != self.mat_type:
            return False
        if other.type_magnetization != self.type_magnetization:
            return False
        if other.Lmag != self.Lmag:
            return False
        return True

    def compare(self, other, name="self"):
        """Compare two objects and return list of differences"""

        if type(other) != type(self):
            return ["type(" + name + ")"]
        diff_list = list()
        if (other.mat_type is None and self.mat_type is not None) or (
            other.mat_type is not None and self.mat_type is None
        ):
            diff_list.append(name + ".mat_type None mismatch")
        elif self.mat_type is not None:
            diff_list.extend(
                self.mat_type.compare(other.mat_type, name=name + ".mat_type")
            )
        if other._type_magnetization != self._type_magnetization:
            diff_list.append(name + ".type_magnetization")
        if other._Lmag != self._Lmag:
            diff_list.append(name + ".Lmag")
        return diff_list

    def __sizeof__(self):
        """Return the size in memory of the object (including all subobject)"""

        S = 0  # Full size of the object
        S += getsizeof(self.mat_type)
        S += getsizeof(self.type_magnetization)
        S += getsizeof(self.Lmag)
        return S

    def as_dict(self, **kwargs):
        """
        Convert this object in a json serializable dict (can be use in __init__).
        Optional keyword input parameter is for internal use only
        and may prevent json serializability.
        """

        Magnet_dict = dict()
        if self.mat_type is None:
            Magnet_dict["mat_type"] = None
        else:
            Magnet_dict["mat_type"] = self.mat_type.as_dict(**kwargs)
        Magnet_dict["type_magnetization"] = self.type_magnetization
        Magnet_dict["Lmag"] = self.Lmag
        # The class name is added to the dict for deserialisation purpose
        Magnet_dict["__class__"] = "Magnet"
        return Magnet_dict

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        if self.mat_type is not None:
            self.mat_type._set_None()
        self.type_magnetization = None
        self.Lmag = None

    def _get_mat_type(self):
        """getter of mat_type"""
        return self._mat_type

    def _set_mat_type(self, value):
        """setter of mat_type"""
        if isinstance(value, str):  # Load from file
            value = load_init_dict(value)[1]
        if isinstance(value, dict) and "__class__" in value:
            class_obj = import_class(
                "pyleecan.Classes", value.get("__class__"), "mat_type"
            )
            value = class_obj(init_dict=value)
        elif type(value) is int and value == -1:  # Default constructor
            value = Material()
        check_var("mat_type", value, "Material")
        self._mat_type = value

        if self._mat_type is not None:
            self._mat_type.parent = self

    mat_type = property(
        fget=_get_mat_type,
        fset=_set_mat_type,
        doc=u"""The Magnet material

        :Type: Material
        """,
    )

    def _get_type_magnetization(self):
        """getter of type_magnetization"""
        return self._type_magnetization

    def _set_type_magnetization(self, value):
        """setter of type_magnetization"""
        check_var("type_magnetization", value, "int", Vmin=0, Vmax=5)
        self._type_magnetization = value

    type_magnetization = property(
        fget=_get_type_magnetization,
        fset=_set_type_magnetization,
        doc=u"""Permanent magnet magnetization type: 0 for radial, 1 for parallel, 2 for Hallbach

        :Type: int
        :min: 0
        :max: 5
        """,
    )

    def _get_Lmag(self):
        """getter of Lmag"""
        return self._Lmag

    def _set_Lmag(self, value):
        """setter of Lmag"""
        check_var("Lmag", value, "float", Vmin=0)
        self._Lmag = value

    Lmag = property(
        fget=_get_Lmag,
        fset=_set_Lmag,
        doc=u"""Magnet axial length

        :Type: float
        :min: 0
        """,
    )



# -*- coding: utf-8 -*-

# File generated according to PHoleM57.ui
# WARNING! All changes made in this file will be lost!
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *

from ......GUI.Tools.FloatEdit import FloatEdit
from ......GUI.Dialog.DMatLib.WMatSelect.WMatSelect import WMatSelect

from pyleecan.GUI.Resources import pyleecan_rc


class Ui_PHoleM57(object):
    def setupUi(self, PHoleM57):
        if not PHoleM57.objectName():
            PHoleM57.setObjectName(u"PHoleM57")
        PHoleM57.resize(731, 470)
        PHoleM57.setMinimumSize(QSize(0, 0))
        PHoleM57.setMaximumSize(QSize(16777215, 16777215))
        self.horizontalLayout = QHBoxLayout(PHoleM57)
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.img_slot = QLabel(PHoleM57)
        self.img_slot.setObjectName(u"img_slot")
        sizePolicy = QSizePolicy(QSizePolicy.Preferred, QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.img_slot.sizePolicy().hasHeightForWidth())
        self.img_slot.setSizePolicy(sizePolicy)
        self.img_slot.setMinimumSize(QSize(0, 0))
        self.img_slot.setMaximumSize(QSize(16777215, 16777215))
        self.img_slot.setPixmap(
            QPixmap(u":/images/images/MachineSetup/SMHoleMag/HoleM57.png")
        )
        self.img_slot.setScaledContents(True)

        self.horizontalLayout.addWidget(self.img_slot)

        self.widget = QWidget(PHoleM57)
        self.widget.setObjectName(u"widget")
        self.widget.setMinimumSize(QSize(250, 0))
        self.widget.setMaximumSize(QSize(250, 16777215))
        self.verticalLayout_2 = QVBoxLayout(self.widget)
        self.verticalLayout_2.setObjectName(u"verticalLayout_2")
        self.gridLayout = QGridLayout()
        self.gridLayout.setObjectName(u"gridLayout")
        self.lf_W3 = FloatEdit(self.widget)
        self.lf_W3.setObjectName(u"lf_W3")

        self.gridLayout.addWidget(self.lf_W3, 5, 1, 1, 1)

        self.lf_W0 = FloatEdit(self.widget)
        self.lf_W0.setObjectName(u"lf_W0")

        self.gridLayout.addWidget(self.lf_W0, 2, 1, 1, 1)

        self.in_W1 = QLabel(self.widget)
        self.in_W1.setObjectName(u"in_W1")

        self.gridLayout.addWidget(self.in_W1, 3, 0, 1, 1)

        self.lf_H2 = FloatEdit(self.widget)
        self.lf_H2.setObjectName(u"lf_H2")

        self.gridLayout.addWidget(self.lf_H2, 1, 1, 1, 1)

        self.unit_W1 = QLabel(self.widget)
        self.unit_W1.setObjectName(u"unit_W1")

        self.gridLayout.addWidget(self.unit_W1, 3, 2, 1, 1)

        self.lf_H1 = FloatEdit(self.widget)
        self.lf_H1.setObjectName(u"lf_H1")

        self.gridLayout.addWidget(self.lf_H1, 0, 1, 1, 1)

        self.unit_W2 = QLabel(self.widget)
        self.unit_W2.setObjectName(u"unit_W2")

        self.gridLayout.addWidget(self.unit_W2, 4, 2, 1, 1)

        self.in_W3 = QLabel(self.widget)
        self.in_W3.setObjectName(u"in_W3")

        self.gridLayout.addWidget(self.in_W3, 5, 0, 1, 1)

        self.unit_W4 = QLabel(self.widget)
        self.unit_W4.setObjectName(u"unit_W4")

        self.gridLayout.addWidget(self.unit_W4, 6, 2, 1, 1)

        self.lf_W4 = FloatEdit(self.widget)
        self.lf_W4.setObjectName(u"lf_W4")

        self.gridLayout.addWidget(self.lf_W4, 6, 1, 1, 1)

        self.unit_W0 = QLabel(self.widget)
        self.unit_W0.setObjectName(u"unit_W0")

        self.gridLayout.addWidget(self.unit_W0, 2, 2, 1, 1)

        self.in_W2 = QLabel(self.widget)
        self.in_W2.setObjectName(u"in_W2")

        self.gridLayout.addWidget(self.in_W2, 4, 0, 1, 1)

        self.in_W0 = QLabel(self.widget)
        self.in_W0.setObjectName(u"in_W0")

        self.gridLayout.addWidget(self.in_W0, 2, 0, 1, 1)

        self.in_W4 = QLabel(self.widget)
        self.in_W4.setObjectName(u"in_W4")

        self.gridLayout.addWidget(self.in_W4, 6, 0, 1, 1)

        self.unit_H1 = QLabel(self.widget)
        self.unit_H1.setObjectName(u"unit_H1")

        self.gridLayout.addWidget(self.unit_H1, 0, 2, 1, 1)

        self.in_H2 = QLabel(self.widget)
        self.in_H2.setObjectName(u"in_H2")

        self.gridLayout.addWidget(self.in_H2, 1, 0, 1, 1)

        self.lf_W2 = FloatEdit(self.widget)
        self.lf_W2.setObjectName(u"lf_W2")

        self.gridLayout.addWidget(self.lf_W2, 4, 1, 1, 1)

        self.in_H1 = QLabel(self.widget)
        self.in_H1.setObjectName(u"in_H1")

        self.gridLayout.addWidget(self.in_H1, 0, 0, 1, 1)

        self.unit_W3 = QLabel(self.widget)
        self.unit_W3.setObjectName(u"unit_W3")

        self.gridLayout.addWidget(self.unit_W3, 5, 2, 1, 1)

        self.lf_W1 = FloatEdit(self.widget)
        self.lf_W1.setObjectName(u"lf_W1")

        self.gridLayout.addWidget(self.lf_W1, 3, 1, 1, 1)

        self.unit_H2 = QLabel(self.widget)
        self.unit_H2.setObjectName(u"unit_H2")

        self.gridLayout.addWidget(self.unit_H2, 1, 2, 1, 1)

        self.verticalLayout_2.addLayout(self.gridLayout)

        self.w_mat_0 = WMatSelect(self.widget)
        self.w_mat_0.setObjectName(u"w_mat_0")
        self.w_mat_0.setMinimumSize(QSize(100, 0))

        self.verticalLayout_2.addWidget(self.w_mat_0)

        self.w_mat_1 = WMatSelect(self.widget)
        self.w_mat_1.setObjectName(u"w_mat_1")
        self.w_mat_1.setMinimumSize(QSize(100, 0))

        self.verticalLayout_2.addWidget(self.w_mat_1)

        self.w_mat_2 = WMatSelect(self.widget)
        self.w_mat_2.setObjectName(u"w_mat_2")
        self.w_mat_2.setMinimumSize(QSize(100, 0))

        self.verticalLayout_2.addWidget(self.w_mat_2)

        self.verticalSpacer_2 = QSpacerItem(
            20, 40, QSizePolicy.Minimum, QSizePolicy.Expanding
        )

        self.verticalLayout_2.addItem(self.verticalSpacer_2)

        self.g_output = QGroupBox(self.widget)
        self.g_output.setObjectName(u"g_output")
        self.g_output.setMinimumSize(QSize(200, 0))
        self.verticalLayout = QVBoxLayout(self.g_output)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.out_slot_surface = QLabel(self.g_output)
        self.out_slot_surface.setObjectName(u"out_slot_surface")

        self.verticalLayout.addWidget(self.out_slot_surface)

        self.out_magnet_surface = QLabel(self.g_output)
        self.out_magnet_surface.setObjectName(u"out_magnet_surface")

        self.verticalLayout.addWidget(self.out_magnet_surface)

        self.verticalLayout_2.addWidget(self.g_output)

        self.horizontalLayout.addWidget(self.widget)

        QWidget.setTabOrder(self.lf_H1, self.lf_H2)
        QWidget.setTabOrder(self.lf_H2, self.lf_W0)
        QWidget.setTabOrder(self.lf_W0, self.lf_W1)
        QWidget.setTabOrder(self.lf_W1, self.lf_W2)
        QWidget.setTabOrder(self.lf_W2, self.lf_W3)
        QWidget.setTabOrder(self.lf_W3, self.lf_W4)

        self.retranslateUi(PHoleM57)

        QMetaObject.connectSlotsByName(PHoleM57)

    # setupUi

    def retranslateUi(self, PHoleM57):
        PHoleM57.setWindowTitle(QCoreApplication.translate("PHoleM57", u"Form", None))
        self.img_slot.setText("")
        self.in_W1.setText(QCoreApplication.translate("PHoleM57", u"W1", None))
        self.unit_W1.setText(QCoreApplication.translate("PHoleM57", u"m", None))
        self.unit_W2.setText(QCoreApplication.translate("PHoleM57", u"m", None))
        self.in_W3.setText(QCoreApplication.translate("PHoleM57", u"W3", None))
        self.unit_W4.setText(QCoreApplication.translate("PHoleM57", u"m", None))
        self.lf_W4.setText("")
        self.unit_W0.setText(QCoreApplication.translate("PHoleM57", u"[rad]", None))
        self.in_W2.setText(QCoreApplication.translate("PHoleM57", u"W2", None))
        self.in_W0.setText(QCoreApplication.translate("PHoleM57", u"W0", None))
        self.in_W4.setText(QCoreApplication.translate("PHoleM57", u"W4", None))
        self.unit_H1.setText(QCoreApplication.translate("PHoleM57", u"m", None))
        self.in_H2.setText(QCoreApplication.translate("PHoleM57", u"H2", None))
        self.in_H1.setText(QCoreApplication.translate("PHoleM57", u"H1", None))
        self.unit_W3.setText(QCoreApplication.translate("PHoleM57", u"m", None))
        self.unit_H2.setText(QCoreApplication.translate("PHoleM57", u"m", None))
        self.g_output.setTitle(QCoreApplication.translate("PHoleM57", u"Output", None))
        self.out_slot_surface.setText(
            QCoreApplication.translate("PHoleM57", u"Slot suface (2 part) : ?", None)
        )
        self.out_magnet_surface.setText(
            QCoreApplication.translate("PHoleM57", u"Single Magnet surface : ?", None)
        )

    # retranslateUi



from PySide2 import QtWidgets, QtGui, QtCore
from matplotlib.backends.backend_qt5agg import FigureCanvas
from matplotlib.pyplot import subplots
from matplotlib.figure import Figure
import numpy as np


DEBUG = True

# =============================================================================
class MPLCanvas(QtWidgets.QDialog):
    def __init__(self, *args, **kwargs):
        QtWidgets.QDialog.__init__(self, *args, **kwargs)
        self.setLayout(QtWidgets.QGridLayout())

        self.fig, self.axes = subplots()
        self.canvas = FigureCanvas(self.fig)
        # 'new' method to be compatible with pyleecan's plot-methods
        self.fig.show = self.canvas.draw

        self.layout().addWidget(self.canvas, 0, 0, 1, 1)


class MPLCanvas2(FigureCanvas):
    def __init__(self, parent=None, width=5, height=4, dpi=100):
        self.fig = Figure()
        self.axes = self.fig.add_subplot(111)
        super(MPLCanvas2, self).__init__(self.fig)

    def refresh_fig(self):
        self.fig = Figure()
        self.axes = self.fig.add_subplot(111)



# -*- coding: utf-8 -*-

from matplotlib.patches import Patch
from matplotlib.pyplot import axis, legend

from ....Functions.init_fig import init_fig
from ....definitions import config_dict

VENT_COLOR = config_dict["PLOT"]["COLOR_DICT"]["VENT_COLOR"]
VENT_EDGE = config_dict["PLOT"]["COLOR_DICT"]["VENT_EDGE"]
ROTOR_COLOR = config_dict["PLOT"]["COLOR_DICT"]["ROTOR_COLOR"]
STATOR_COLOR = config_dict["PLOT"]["COLOR_DICT"]["STATOR_COLOR"]


def plot(
    self,
    fig=None,
    ax=None,
    is_lam_only=False,
    sym=1,
    alpha=0,
    delta=0,
    is_edge_only=False,
    is_show_fig=True,
):
    """Plot the Lamination in a matplotlib fig

    Parameters
    ----------
    self : Lamination
        A Lamination object
    fig : Matplotlib.figure.Figure
        existing figure to use if None create a new one
    ax : Matplotlib.axes.Axes object
        Axis on which to plot the data
    is_lam_only: bool
        True to plot only the lamination (no effect for Lamination object)
    sym : int
        Symmetry factor (1= full machine, 2= half of the machine...)
    alpha : float
        Angle for rotation [rad]
    delta : complex
        Complex value for translation
    is_edge_only: bool
        To plot transparent Patches
    is_show_fig : bool
        To call show at the end of the method

    Returns
    -------

    """
    if self.is_stator:
        lam_color = STATOR_COLOR
    else:
        lam_color = ROTOR_COLOR
    (fig, axes, patch_leg, label_leg) = init_fig(fig=fig, ax=ax, shape="rectangle")

    surf_list = self.build_geometry(sym=sym, alpha=alpha, delta=delta)
    patches = list()

    # Color Selection for Lamination
    for surf in surf_list:
        if surf.label is not None and "Lamination" in surf.label:
            patches.extend(surf.get_patches(color=lam_color, is_edge_only=is_edge_only))
        elif surf.label is not None and "Ventilation_" in surf.label:
            patches.extend(
                surf.get_patches(
                    color=VENT_COLOR, edgecolor=VENT_EDGE, is_edge_only=is_edge_only
                )
            )
        else:
            patches.extend(surf.get_patches(is_edge_only=is_edge_only))

    # Display the result
    axes.set_xlabel("(m)")
    axes.set_ylabel("(m)")
    for patch in patches:
        axes.add_patch(patch)

    # Axis Setup
    axes.axis("equal")

    # The Lamination is centered in the figure
    Lim = self.Rext * 1.5
    axes.set_xlim(-Lim, Lim)
    axes.set_ylim(-Lim, Lim)

    # Adding legend
    if not is_edge_only:
        if self.is_stator:
            patch_leg.append(Patch(color=STATOR_COLOR))
            label_leg.append("Stator")
            axes.set_title("Stator without slot")
        else:
            patch_leg.append(Patch(color=ROTOR_COLOR))
            label_leg.append("Rotor")
            axes.set_title("Rotor without slot")

        legend(patch_leg, label_leg)
    if is_show_fig:
        fig.show()



from PySide2.QtCore import Qt, Signal
from PySide2.QtWidgets import QWidget, QMessageBox, QDialog
from pandas import read_excel, ExcelFile
from .....Classes.ImportMatrixXls import ImportMatrixXls
from .....Classes.ImportMatrixVal import ImportMatrixVal
from .....GUI.Tools.WImport.WImportExcel.Ui_WImportExcel import Ui_WImportExcel
from .....GUI.Tools.WTableData.WTableData import DTableData
from os.path import isfile
import matplotlib.pyplot as plt


class WImportExcel(Ui_WImportExcel, QWidget):
    """Widget to define an ImportMatrixXls"""

    import_name = "Import from Excel"
    import_type = ImportMatrixXls
    saveNeeded = Signal()
    dataTypeChanged = Signal()

    def __init__(self, parent=None, data=None, verbose_name="", expected_shape=None):
        """Initialization of the widget

        Parameters
        ----------
        data : ImportMatrixXls
            Data import to define
        verbose_name : str
            Name of the imported data
        expected_shape : list
            List to enforce a shape, [None, 2] enforce 2D matrix with 2 columns
        """
        QWidget.__init__(self, parent=parent)
        self.setupUi(self)

        if data is None or not isinstance(data, ImportMatrixXls):
            self.data = ImportMatrixXls()
        else:
            self.data = data
        self.verbose_name = verbose_name
        self.expected_shape = expected_shape

        # Not used yet
        self.g_axe1.hide()
        self.g_axe2.hide()

        # Setup Path selector
        self.w_file_path.is_file = True
        self.w_file_path.obj = self.data
        self.w_file_path.verbose_name = "Excel file path"
        self.w_file_path.param_name = "file_path"
        self.w_file_path.extension = "Excel File (*.xls *.xlsx)"
        # Display current state of ImportMatrixXls
        self.update()

        # Connect the slot/signal
        self.w_file_path.pathChanged.connect(self.set_sheet_list)
        self.c_sheet.currentIndexChanged.connect(self.set_sheet)
        self.le_range.editingFinished.connect(self.set_range)
        self.b_plot.clicked.connect(self.s_plot)
        self.b_tab.clicked.connect(self.s_table)
        self.b_convert.clicked.connect(self.s_convert)

    def update(self):
        """Fill the widget with the current value of the data"""
        self.w_file_path.update()
        self.set_sheet_list()
        self.le_range.setText(self.data.usecols)

    def set_sheet_list(self):
        """Complete the combobox with the sheet name from the Excel file"""
        if is_excel_file(self.data.file_path):
            # Get Sheet names
            xls_file = ExcelFile(self.data.file_path)
            # Disable combobox if only one sheet
            if len(xls_file.sheet_names) > 1:
                self.c_sheet.setEnabled(True)
            else:
                self.c_sheet.setEnabled(False)
            self.c_sheet.blockSignals(True)
            # Fill combobox
            self.c_sheet.clear()
            for name in xls_file.sheet_names:
                self.c_sheet.addItem(name)
            # Select current index
            if (
                self.data.sheet not in ["", None]
                and self.data.sheet in xls_file.sheet_names
            ):
                self.c_sheet.setCurrentText(self.data.sheet)
            else:
                self.data.sheet = xls_file.sheet_names[0]
                self.c_sheet.setCurrentIndex(0)
            self.c_sheet.blockSignals(False)

    def set_sheet(self):
        """Change the current sheet selected"""
        self.data.sheet = self.c_sheet.currentText()

    def set_range(self):
        """Change the range of data selected"""
        self.data.usecols = self.le_range.text()
        if self.data.usecols == "":
            self.data.usecols = None

    def s_table(self):
        """display the data in a table"""
        try:
            data = self.data.get_data()
        except Exception as e:
            QMessageBox.critical(self, self.tr("Error"), str(e))
            return
        tab = DTableData(data=data)
        return_code = tab.exec_()

    def s_plot(self):
        """display the data in a plot"""
        try:
            data = self.data.get_data()
        except Exception as e:
            QMessageBox.critical(self, self.tr("Error"), str(e))
            return
        if len(data.shape) == 2 and data.shape[0] == 2:
            # Data in line
            fig, axes = plt.subplots()
            axes.plot(data[0, :], data[1, :])
            fig.show()
        elif len(data.shape) == 2 and data.shape[1] == 2:
            # Data in column
            fig, axes = plt.subplots()
            axes.plot(data[:, 0], data[:, 1])
            fig.show()

    def s_convert(self):
        """Convert data to ImportMatrixVal"""
        try:
            data = self.data.get_data()
        except Exception as e:
            QMessageBox.critical(self, self.tr("Error"), str(e))
            return
        self.data = ImportMatrixVal(value=data)
        self.dataTypeChanged.emit()


def is_excel_file(file_path):
    """Check if the path is correct for an excel file"""

    return (
        file_path is not None
        and isfile(file_path)
        and (file_path[-4:] == ".xls" or file_path[-5:] == ".xlsx")
    )



from ....Methods.Machine.Lamination import LaminationCheckError


class Lam_WindCheckError(LaminationCheckError):
    """ """

    pass


class LWC_OutSlotInLam(Lam_WindCheckError):
    """ """

    pass


class LWC_InSlotOutLam(Lam_WindCheckError):
    """ """

    pass


class LWC_SlotTooLong(Lam_WindCheckError):
    """ """

    pass


class LWC_OverlappingSlot(Lam_WindCheckError):
    """ """

    pass



# -*- coding: utf-8 -*-


def get_data(self):
    """Return the object's matrix

    Parameters
    ----------
    self : ImportMatrixVal
        An ImportMatrixVal object

    Returns
    -------
    matrix: ndarray
        The object's matrix

    """

    return self.edit_matrix(self.value)



# -*- coding: utf-8 -*-
from ....Methods.Slot.VentilationPolar import *


def check(self):
    """Make sure that the ventilation parameters are correctly set

    Parameters
    ----------
    self : VentilationPolar
        A VentilationPolar object

    Returns
    -------
    None

    Raises
    _______
    VentilationPolarInstanceError
        Zh must be a integer
        H0 must be a float or int
        D0 must be a float or int
        Alpha0 must be a float or int
    """

    if not isinstance(self.Zh, int):
        raise VentilationPolarInstanceError("Zh must be a integer")
    if not isinstance(self.H0, float) and not isinstance(self.H0, int):
        raise VentilationPolarInstanceError("H0 must be a float or int")
    if not isinstance(self.D0, float) and not isinstance(self.D0, int):
        raise VentilationPolarInstanceError("D0 must be a float or int")
    if not isinstance(self.Alpha0, float) and not isinstance(self.Alpha0, int):
        raise VentilationPolarInstanceError("Alpha0 must be a float or int")



from ....Classes.DataKeeper import DataKeeper


def get_elec_datakeeper(self, symbol_list):
    """
    Generate DataKeepers to store by default results from electric module

    Parameters
    ----------
    self: VarLoad
        A VarLoad object
    symbol_list : list
        List of the existing datakeeper (to avoid duplicate)

    Returns
    -------
    dk_list: list
        list of DataKeeper
    """
    dk_list = []
    # Save Id
    if "Id" not in symbol_list:
        dk_list.append(
            DataKeeper(
                name="Id",
                symbol="Id",
                unit="Arms",
                keeper="lambda output: output.elec.Id_ref",
            )
        )
    # Save Iq
    if "Iq" not in symbol_list:
        dk_list.append(
            DataKeeper(
                name="Iq",
                symbol="Iq",
                unit="Arms",
                keeper="lambda output: output.elec.Iq_ref",
            )
        )

    return dk_list



# -*- coding: utf-8 -*-

from numpy import sqrt


def comp_radius(self):
    """Compute the radius of the two circle that contains all the ventilation
    ducts

    Parameters
    ----------
    self : VentilationTrap
        A VentilationTrap object

    Returns
    -------
    (Rmin, Rmax): tuple
        Tuple of circle radius [m]

    """

    self.check()

    Rmin = sqrt(self.W1 ** 2 + self.H0 ** 2)
    Rmax = sqrt(self.W2 ** 2 + (self.H0 + self.D0) ** 2)

    return (Rmin, Rmax)



def get_simu(self, index):
    """
    Generate a simulation according to index

    Parameters
    ----------
    index: int
        simulation index

    Returns
    -------
    simu : Simulation
        simulation generated
    """

    # Get reference simulation
    ref_simu = self.simu

    # Construct the simulation
    simu = ref_simu.copy()
    simu.var_simu = None

    # Run setters
    for paramexplorer in self.paramexplorer_list:
        paramexplorer.setter(simu, paramexplorer.value[index])

    return simu
